/*
 * $Id: Thread.cpp,v 1.2 2007/06/26 14:46:20 bacon Exp $
 */

#include <xpp/bas/Thread.hpp>
#include <xp/bas/time.h>
#include <xp/bas/assert.h>

namespace axp
{
	Thread::Handle Thread::INVALID_HANDLE = XP_THRHND_INVALID;

	Thread::Thread (): thread_target (XP_NULL)
	{
		this->user_data = user_data;
		this->handle = Thread::INVALID_HANDLE;
		this->state = Thread::INCUBATING;
		this->main_routine = AXP_NULL;
		this->joinable = true;
	}

	Thread::~Thread ()
	{
		xp_assert (this->__state != Thread::RUNNING);
		// it is subclasses' responsibility to stop the thread gracefully.
		// so stop is not called here.
		// stop ();

		/*if (this->__joinable)*/ join ();
	#ifdef _WIN32
		if (this->handle != Thread::INVALID_HANDLE)
		{
			::CloseHandle (this->handle);
			this->handle = Thread::INVALID_HANDLE;
		}
	#endif
	}

	Thread::Handle Thread::current ()
	{
	#if defined(_WIN32)
		return ::GetCurrentThread ();
	#elif defined(__BEOS__)
		return Thread::INVALID_HANDLE; /* TODO: implement this */
	#else
		return pthread_self ();
	#endif
	}

	int Thread::start (int flags)
	{
		if (this->__state == Thread::RUNNING) return -1;
		
		this->__joinable = ((flags & Thread::DETACHED) == 0);
		if (create_thread() == -1) 
		{
			this->__state = Thread::INCUBATING;
			this->__handle = Thread::INVALID_HANDLE;
			return -1;
		}

		this->__state = Thread::RUNNING;
		return 0;
	}

	int Thread::stop ()
	{
		// NOTICE:
		// make sure that subclasses override "stop" and call it
		// properly so that the thread can be terminated gracefully.
		// "stop" here just aborts the running thread.
		return xp_thr_stop (this);
	}

	int Thread::create_thread ()
	{
	#ifdef _WIN32
		unsigned int tid;
		if (this->__handle != Thread::INVALID_HANDLE) {
			::CloseHandle (this->__handle);
		}
		this->__handle = (HANDLE)::_beginthreadex (XP_NULL, 0,
			(unsigned int (__stdcall*)(void*))Thread::start_thread,
			this, 0, &tid);
		if (this->__handle == 0) return -1;
	#else
		pthread_attr_t attr;
		::pthread_attr_init (&attr);

		if (this->__joinable) {
			if (::pthread_attr_setdetachstate (
				&attr, PTHREAD_CREATE_JOINABLE) != 0) return -1;
		}
		else {
			if (::pthread_attr_setdetachstate (
				&attr, PTHREAD_CREATE_DETACHED) != 0) return -1;
		}

		if (::pthread_create (
			&this->__handle, &attr, Thread::start_thread, this) != 0) {
			::pthread_attr_destroy (&attr);
			return -1;
		}
		::pthread_attr_destroy (&attr);
	#endif

		return 0;
	}

	void* Thread::start_thread (void* arg)
	{
		Thread* t = (Thread*)arg;

		// Prevent the thread from running before "thread_state" is set
		// to RUNNING in "start". Otherwise, "start_thread" can terminate
		// (thread_state is set to TERMINATED at this point) before
		// "thread_state" is set to RUNNING in "start" so "thread_state"
		// becomes RUNNING even after the thread terminates.
		while (t->__state != Thread::RUNNING) xp_sleep (100);

#ifndef _WIN32
		::pthread_setcancelstate (PTHREAD_CANCEL_ENABLE, (int*)XP_NULL);
		::pthread_setcanceltype (PTHREAD_CANCEL_ASYNCHRONOUS, (int*)XP_NULL);
#endif
		try {
			t->__return_code =
				(t->thread_target != XP_NULL)?
				t->thread_target->run(): t->run ();
			t->__state = Thread::TERMINATED;
		}
		catch (...) {
			t->__state = Thread::ABORTED;
			// the exception is not thrown futhur.
			// and t->__return_code becomes undefined.
		}

#ifdef _WIN32
		_endthreadex (t->__return_code);
#else
		pthread_exit (&t->__return_code);
#endif
		return XP_NULL;
	}

}

