/*
 * $Id: MysqlStorage.java,v 1.14 2007/05/19 04:43:59 bacon Exp $
 */

package com.unimar.shipneeds;

import java.util.List;
import java.util.LinkedList;
import java.util.Iterator;
import java.sql.*;

public class MysqlStorage extends Storage
{
	private String url;
	private Connection conn;

	static 
	{
		try { Class.forName("com.mysql.jdbc.Driver"); }
		catch (ClassNotFoundException e)  
		{ 
			// TODO: BETTER EXCEPTION HANDLING 
		}
	}

	public MysqlStorage ()
	{
		// TODO: read from configuration...
		url = "jdbc:mysql://localhost:3306/shipneeds?useUnicode=true&characterEncoding=utf8";
	}

	public MysqlStorage (String host, int port, String dbname)
	{
		url = "jdbc:mysql://" + host + ":" + 
			Integer.toString(port) + "/" + dbname +
			"?useUnicode=true&characterEncoding=utf8";
	}

	private void connect () throws StorageException
	{
		if (conn != null) return;

		// TODO: get credential from somewhere else... 
		try 
		{ 
			conn = DriverManager.getConnection (url, "root", ""); 
			conn.setAutoCommit (false);
		}
		catch (SQLException e) 
		{ 
			throw new StorageException(e.getMessage());
		}
	}

	public void close ()
	{
		if (conn != null) 
		{
			try { conn.rollback (); }
			catch (SQLException e) {}

			try { conn.close (); }
			catch (SQLException e) {}

			conn = null;
		}
	}

	private void closeStatement (Statement stmt)
	{
		if (stmt == null) return;
		try { stmt.close (); } catch (SQLException e) {}
	}

	public synchronized void addGroup (Group group) throws StorageException
	{
		connect ();

		PreparedStatement stmt = null;

		try
		{
			stmt = conn.prepareStatement (
				"INSERT INTO groups(groupname,longname,email,address,telno1,telno2,telno3,rolename,intrinsic) VALUES(?,?,?,?,?,?,?,?,'N')");

			stmt.setString (1, group.getName());
			stmt.setString (2, group.getLongName());
			stmt.setString (3, group.getEmail());
			stmt.setString (4, group.getAddress());
			stmt.setString (5, group.getTelno1());
			stmt.setString (6, group.getTelno2());
			stmt.setString (7, group.getTelno3());
			stmt.setString (8, group.getRole().getName());
			stmt.executeUpdate ();
			closeStatement (stmt); stmt = null;

			conn.commit ();
		}
		catch (SQLException e)
		{
			close ();
			throw new StorageException  (e.getMessage());
		}
		finally 
		{
			closeStatement (stmt);
		}

	}

	public synchronized void updateGroup (
		Group oldGroup, Group newGroup) throws StorageException
	{
		connect ();

		PreparedStatement stmt = null;

		try
		{
			stmt = conn.prepareStatement (
				"UPDATE groups SET groupname=?, longname=?, email=?, address=?, telno1=?, telno2=?, telno3=?, rolename=? WHERE groupname=?");

			stmt.setString (1, newGroup.getName());
			stmt.setString (2, newGroup.getLongName());
			stmt.setString (3, newGroup.getEmail());
			stmt.setString (4, newGroup.getAddress());
			stmt.setString (5, newGroup.getTelno1());
			stmt.setString (6, newGroup.getTelno2());
			stmt.setString (7, newGroup.getTelno3());
			stmt.setString (8, newGroup.getRole().getName());
			stmt.setString (9, oldGroup.getName());
			stmt.executeUpdate ();
			closeStatement (stmt); stmt = null;

			stmt = conn.prepareStatement (
				"UPDATE user_roles SET rolename=? WHERE username IN (SELECT username FROM users WHERE groupname=?)");

			stmt.setString(1, newGroup.getRole().getName());
			stmt.setString(2, newGroup.getName());
			stmt.executeUpdate ();
			closeStatement (stmt); stmt = null;

			conn.commit ();
		}
		catch (SQLException e)
		{
			close ();
			throw new StorageException  (e.getMessage());
		}
		finally 
		{
			closeStatement (stmt);
		}
	}

	public synchronized void deleteGroup (String group) throws StorageException
	{
		connect ();

		PreparedStatement stmt = null;

		try
		{

			stmt = conn.prepareStatement (
				"DELETE FROM user_roles WHERE username IN (SELECT username FROM users WHERE groupname=?)");
			stmt.setString (1, group);
			stmt.executeUpdate ();
			closeStatement (stmt); stmt = null;

			stmt = conn.prepareStatement ("DELETE FROM users WHERE groupname=?");
			stmt.setString (1, group);
			stmt.executeUpdate ();
			closeStatement (stmt); stmt = null;

			stmt = conn.prepareStatement ("DELETE FROM groups WHERE groupname=?");
			stmt.setString (1, group);
			stmt.executeUpdate ();
			closeStatement (stmt); stmt = null;

			conn.commit ();
		}
		catch (SQLException e)
		{
			close ();
			throw new StorageException  (e.getMessage());
		}
		finally 
		{
			closeStatement (stmt);
		}
	}

	public synchronized List getGroups () throws StorageException
	{
		connect ();

		LinkedList list = new LinkedList();

		Statement stmt = null;

		try
		{
			stmt = conn.createStatement ();

			ResultSet rs = stmt.executeQuery (
				"SELECT groupname,groups.longname,email,address,telno1,telno2,telno3,groups.rolename,roles.longname FROM groups, roles WHERE groups.rolename = roles.rolename AND groups.intrinsic = 'N' AND roles.intrinsic='N'");

			while (rs.next())
			{
				Group g = new Group ();

				g.setName (rs.getString("groupname"));
				g.setLongName (rs.getString("groups.longname"));
				g.setEmail (rs.getString("email"));
				g.setAddress (rs.getString("address"));
				g.setTelno1 (rs.getString("telno1"));
				g.setTelno2 (rs.getString("telno2"));
				g.setTelno3 (rs.getString("telno3"));

				Role r = new Role ();
				r.setName (rs.getString("rolename"));
				r.setLongName (rs.getString("roles.longname"));
				g.setRole (r);

				list.addLast (g);
			}

			closeStatement (stmt); stmt = null;
			conn.commit ();
		}
		catch (SQLException e)
		{
			close ();
			throw new StorageException  (e.getMessage());
		}
		finally
		{
			closeStatement (stmt);
		}

		return list;
	}

	public synchronized void addUser (User user) throws StorageException
	{
		connect ();

		PreparedStatement stmt = null;

		try
		{
			stmt = conn.prepareStatement (
				"INSERT INTO users(username,longname,password,groupname,email,intrinsic) VALUES(?,?,?,?,?,'N')");

			stmt.setString (1, user.getName());
			stmt.setString (2, user.getLongName());
			stmt.setString (3, user.getPassword());
			stmt.setString (4, user.getGroup());
			stmt.setString (5, user.getEmail());
			stmt.executeUpdate ();
			closeStatement (stmt); stmt = null;

			stmt = conn.prepareStatement ("INSERT INTO user_roles(username,rolename) VALUES(?,?)");
			stmt.setString (1, user.getName());
			stmt.setString (2, user.getRole().getName());
			stmt.executeUpdate ();
			closeStatement (stmt); stmt = null;

			conn.commit ();
		}
		catch (SQLException e)
		{
			close ();
			throw new StorageException  (e.getMessage());
		}
		finally
		{
			closeStatement (stmt);
		}

	}

	public synchronized void updateUser (
		User oldUser, User newUser) throws StorageException
	{
		connect ();

		PreparedStatement stmt = null;

		try
		{
			stmt = conn.prepareStatement (
				"UPDATE users SET username=?, longname=?, password=?, groupname=?, email=? WHERE username=?");

			stmt.setString (1, newUser.getName());
			stmt.setString (2, newUser.getLongName());
			stmt.setString (3, newUser.getPassword());
			stmt.setString (4, newUser.getGroup());
			stmt.setString (5, newUser.getEmail());
			stmt.setString (6, oldUser.getName());
			stmt.executeUpdate ();
			closeStatement (stmt); stmt = null;

			stmt = conn.prepareStatement ("UPDATE user_roles SET username=?,rolename=? WHERE username=?");
			stmt.setString (1, newUser.getName());
			stmt.setString (2, newUser.getRole().getName());
			stmt.setString (3, oldUser.getName());
			closeStatement (stmt); stmt = null;

			conn.commit ();
		}
		catch (SQLException e)
		{
			close ();
			throw new StorageException  (e.getMessage());
		}
		finally 
		{
			closeStatement (stmt);
		}
	}

	public synchronized void deleteUser (String user) throws StorageException
	{
		connect ();

		PreparedStatement stmt = null;

		try
		{
			stmt = conn.prepareStatement ("DELETE FROM users WHERE username=?");
			stmt.setString (1, user);
			stmt.executeUpdate ();
			stmt.close ();
			closeStatement (stmt); stmt = null;

			stmt = conn.prepareStatement ("DELETE FROM user_roles WHERE username=?");
			stmt.setString (1, user);
			stmt.executeUpdate ();
			closeStatement (stmt); stmt = null;

			conn.commit ();
		}
		catch (SQLException e)
		{
			close ();
			throw new StorageException  (e.getMessage());
		}
		finally 
		{
			closeStatement (stmt);
		}
	}

	public synchronized List getUsers (String group) throws StorageException
	{
		connect ();

		LinkedList list = new LinkedList();
		PreparedStatement stmt = null;

		try
		{
			// ensure that a user is only associated a single 
			// role for the shipneeds application. otherwise,
			// the following sql statements will return a lot
			// of duplicate entries.
			stmt = conn.prepareStatement (
				"SELECT users.username,users.longname,password,groupname,email,roles.rolename,roles.longname FROM users, roles, user_roles WHERE groupname=? AND users.intrinsic='N' AND roles.intrinsic='N' AND user_roles.username=users.username AND user_roles.rolename=roles.rolename");
			stmt.setString (1, group);

			ResultSet rs = stmt.executeQuery ();

			while (rs.next())
			{
				User u = new User ();

				u.setName (rs.getString("username"));
				u.setLongName (rs.getString("users.longname"));
				u.setPassword (rs.getString("password"));
				u.setEmail (rs.getString("email"));
				u.setGroup (rs.getString("groupname"));

				Role r = new Role ();
				r.setName (rs.getString("rolename"));
				r.setLongName (rs.getString("roles.longname"));
				u.setRole (r);

				list.addLast (u);
			}

			closeStatement (stmt); stmt = null;
			conn.commit ();
		}
		catch (SQLException e)
		{
			close ();
			throw new StorageException  (e.getMessage());
		}
		finally 
		{
			closeStatement (stmt);
		}

		return list;
	}

	public synchronized List getRoles () throws StorageException
	{
		connect ();
		LinkedList list = new LinkedList();

		Statement stmt = null;

		try
		{
			stmt = conn.createStatement ();
			ResultSet rs = stmt.executeQuery ("SELECT rolename,longname FROM roles WHERE intrinsic = 'N'");

			while (rs.next())
			{
				Role r = new Role ();

				r.setName (rs.getString("rolename"));
				r.setLongName (rs.getString("longname"));

				list.addLast (r);
			}

			closeStatement (stmt); stmt = null;
			conn.commit ();
		}
		catch (SQLException e)
		{
			close ();
			throw new StorageException  (e.getMessage());
		}
		finally 
		{
			closeStatement (stmt);
		}

		return list;
	}


	public synchronized void addRfq (String username, Rfq rfq) throws StorageException
	{
		connect ();

		PreparedStatement stmt = null;

		try
		{
			stmt = conn.prepareStatement ("SELECT groupname FROM users WHERE username=?");
			stmt.setString (1, username);

			ResultSet rs = stmt.executeQuery ();
			if (!rs.next())
			{
				conn.rollback ();
				throw new StorageException ("unknown user - " + username);
			}

			String groupname = rs.getString("groupname");
			closeStatement (stmt); stmt = null;

			stmt = conn.prepareStatement (
				"INSERT INTO rfqs(time,buyer,revno,title,preferred_supplier1,preferred_supplier2,preferred_supplier3,comment,owner,status,stamp) VALUES(?,?,?,?,?,?,?,?,?,?,?)");

			stmt.setLong    (1, rfq.getTime());
			stmt.setString  (2, groupname);
			stmt.setInt     (3, rfq.getRevno());
			stmt.setString  (4, rfq.getTitle());
			stmt.setString  (5, rfq.getPreferredSupplier1());
			stmt.setString  (6, rfq.getPreferredSupplier2());
			stmt.setString  (7, rfq.getPreferredSupplier3());
			stmt.setString  (8, rfq.getComment());
			stmt.setString  (9, username);
			stmt.setString  (10, rfq.getStatus());
			stmt.setLong    (11, rfq.getStamp());
			stmt.executeUpdate ();

			rs = stmt.getGeneratedKeys();
			closeStatement (stmt); stmt = null;

			long rfqno;
			if (!rs.next())
			{
				conn.rollback ();
				throw new StorageException ("cannot get rfqno");
			} 

			rfqno = rs.getLong (1);
			
			Iterator i = rfq.getItems().iterator();
			int seqno = 1;
			while (i.hasNext())
			{
				RfqItem item = (RfqItem)i.next();

				stmt = conn.prepareStatement (
					"INSERT INTO rfqitems(rfqno,seqno,longname,code,preferred_brand,quantity) VALUES(?,?,?,?,?,?)");

				stmt.setLong   (1, rfqno);
				stmt.setInt    (2, seqno++);
				stmt.setString (3, item.getLongName());
				stmt.setString (4, item.getCode());
				stmt.setString (5, item.getPreferredBrand());
				stmt.setInt    (6, item.getQuantity());
				int nrows = stmt.executeUpdate ();

				closeStatement (stmt); stmt = null;

				if (nrows != 1)
				{
					conn.rollback ();
					throw new StorageException ("cannot update");
				}
			}

			conn.commit ();
		}
		catch (SQLException e)
		{
			close ();
			throw new StorageException  (e.getMessage());
		}
		finally
		{
			closeStatement (stmt);
		}
	}

	public synchronized void updateRfq (
		String username, long oldStamp, Rfq rfq) throws StorageException
	{
		connect ();

		PreparedStatement stmt = null;

		try
		{
			stmt = conn.prepareStatement ("SELECT groupname FROM users WHERE username=?");
			stmt.setString (1, username);

			ResultSet rs = stmt.executeQuery ();
			if (!rs.next())
			{
				conn.rollback ();
				throw new StorageException ("unknown user - " + username);
			}

			String groupname = rs.getString("groupname");
			closeStatement (stmt); stmt = null;

			long rfqno = rfq.getRfqno ();

			stmt = conn.prepareStatement (
				"UPDATE rfqs SET title=?,preferred_supplier1=?,preferred_supplier2=?,preferred_supplier3=?,comment=?,status=?,stamp=? WHERE rfqno=? AND buyer=? AND stamp=?");

			stmt.setString  (1, rfq.getTitle());
			stmt.setString  (2, rfq.getPreferredSupplier1());
			stmt.setString  (3, rfq.getPreferredSupplier2());
			stmt.setString  (4, rfq.getPreferredSupplier3());
			stmt.setString  (5, rfq.getComment());
			stmt.setString  (6, rfq.getStatus());
			stmt.setLong    (7, rfq.getStamp());
			stmt.setLong    (8, rfqno);
			stmt.setString  (9, groupname);
			stmt.setLong    (10, oldStamp);
			int nrows = stmt.executeUpdate ();
			closeStatement (stmt); stmt = null;

			if (nrows != 1) 
			{
				conn.rollback ();
				throw new StorageException ("cannot update");
			}

			stmt = conn.prepareStatement ("DELETE FROM rfqitems WHERE rfqno=?");
			stmt.setLong (1, rfqno);
			stmt.executeUpdate ();
			closeStatement (stmt); stmt = null;

			Iterator i = rfq.getItems().iterator();
			int seqno = 1;
			while (i.hasNext())
			{
				RfqItem item = (RfqItem)i.next();

				stmt = conn.prepareStatement (
					"INSERT INTO rfqitems(rfqno,seqno,longname,code,preferred_brand,quantity) VALUES(?,?,?,?,?,?)");

				stmt.setLong   (1, rfqno);
				stmt.setInt    (2, seqno++);
				stmt.setString (3, item.getLongName());
				stmt.setString (4, item.getCode());
				stmt.setString (5, item.getPreferredBrand());
				stmt.setInt    (6, item.getQuantity());
				nrows = stmt.executeUpdate ();

				closeStatement (stmt); stmt = null;

				if (nrows != 1)
				{	
					conn.rollback ();
					throw new StorageException ("cannot update");
				}
			}

			conn.commit ();
		}
		catch (SQLException e)
		{
			close ();
			throw new StorageException  (e.getMessage());
		}
		finally
		{
			closeStatement (stmt);
		}
	}

	public synchronized List getRfqs (String username) throws StorageException
	{
		connect ();

		LinkedList rfqList = new LinkedList();

		PreparedStatement stmt = null;
		PreparedStatement stmt2 = null;

		try
		{
			stmt = conn.prepareStatement ("SELECT groupname FROM users WHERE username=?");
			stmt.setString (1, username);

			ResultSet rs = stmt.executeQuery ();
			if (!rs.next())
			{
				conn.rollback ();
				throw new StorageException ("unknown user - " + username);
			}

			String groupname = rs.getString("groupname");
			closeStatement (stmt); stmt = null;

			stmt = conn.prepareStatement (
				"SELECT rfqno,time,buyer,revno,title,preferred_supplier1,preferred_supplier2,preferred_supplier3,comment,owner,status,stamp FROM rfqs WHERE buyer=?");
			stmt.setString (1, groupname);

			rs = stmt.executeQuery ();

			while (rs.next())
			{
				Rfq r = new Rfq ();

				long rfqno = rs.getLong(1);

				r.setRfqno (rfqno);
				r.setTime (rs.getLong(2));
				r.setBuyer (rs.getString(3));
				r.setRevno (rs.getInt(4));
				r.setTitle (rs.getString(5));
				r.setPreferredSupplier1 (rs.getString(6));
				r.setPreferredSupplier2 (rs.getString(7));
				r.setPreferredSupplier3 (rs.getString(8));
				r.setComment (rs.getString(9));
				r.setOwner (rs.getString(10));
				r.setStatus (rs.getString(11));
				r.setStamp (rs.getLong(12));

				stmt2 = conn.prepareStatement (
					"SELECT seqno,longname,code,preferred_brand,quantity FROM rfqitems WHERE rfqno=?");
				stmt2.setLong (1, rfqno);

				LinkedList itmList = new LinkedList ();
				ResultSet rs2 = stmt2.executeQuery ();
				while (rs2.next())
				{
					RfqItem itm = new RfqItem ();

					itm.setSeqno (rs2.getInt(1));
					itm.setLongName (rs2.getString(2));
					itm.setCode (rs2.getString(3));
					itm.setPreferredBrand (rs2.getString(4));
					itm.setQuantity (rs2.getInt(5));

					itmList.addLast (itm);
				}
				
				r.setItems (itmList);
				rfqList.addLast (r);
			}

			closeStatement (stmt2); stmt2 = null;
			closeStatement (stmt); stmt = null;

			conn.commit ();
		}
		catch (SQLException e)
		{
			close ();
			throw new StorageException  (e.getMessage());
		}
		finally
		{
			closeStatement (stmt2);
			closeStatement (stmt);
		}

		return rfqList;
	}

	public synchronized Rfq getRfq (String username, long rfqno) throws StorageException
	{
		Rfq r = null;

		connect ();

		LinkedList rfqList = new LinkedList();

		PreparedStatement stmt = null;
		PreparedStatement stmt2 = null;

		try
		{
			stmt = conn.prepareStatement ("SELECT groupname FROM users WHERE username=?");
			stmt.setString (1, username);

			ResultSet rs = stmt.executeQuery ();
			if (!rs.next())
			{
				conn.rollback ();
				throw new StorageException ("unknown user - " + username);
			}

			String groupname = rs.getString("groupname");
			closeStatement (stmt); stmt = null;

			stmt = conn.prepareStatement (
				"SELECT rfqno,time,buyer,revno,title,preferred_supplier1,preferred_supplier2,preferred_supplier3,comment,owner,status,stamp FROM rfqs WHERE rfqno=? AND buyer=?");
			stmt.setLong (1, rfqno);
			stmt.setString (2, groupname);

			rs = stmt.executeQuery ();

			if (rs.next())
			{
				r = new Rfq ();

				r.setRfqno (rfqno);
				r.setTime (rs.getLong(2));
				r.setBuyer (rs.getString(3));
				r.setRevno (rs.getInt(4));
				r.setTitle (rs.getString(5));
				r.setPreferredSupplier1 (rs.getString(6));
				r.setPreferredSupplier2 (rs.getString(7));
				r.setPreferredSupplier3 (rs.getString(8));
				r.setComment (rs.getString(9));
				r.setOwner (rs.getString(10));
				r.setStatus (rs.getString(11));
				r.setStamp (rs.getLong(12));

				stmt2 = conn.prepareStatement (
					"SELECT seqno,longname,code,preferred_brand,quantity FROM rfqitems WHERE rfqno=?");
				stmt2.setLong (1, rfqno);

				LinkedList itmList = new LinkedList ();
				ResultSet rs2 = stmt2.executeQuery ();
				while (rs2.next())
				{
					RfqItem itm = new RfqItem ();

					itm.setSeqno (rs2.getInt(1));
					itm.setLongName (rs2.getString(2));
					itm.setCode (rs2.getString(3));
					itm.setPreferredBrand (rs2.getString(4));
					itm.setQuantity (rs2.getInt(5));

					itmList.addLast (itm);
				}
				
				r.setItems (itmList);
			}

			closeStatement (stmt2); stmt2 = null;
			closeStatement (stmt); stmt = null;

			conn.commit ();
		}
		catch (SQLException e)
		{
			close ();
			throw new StorageException  (e.getMessage());
		}
		finally
		{
			closeStatement (stmt2);
			closeStatement (stmt);
		}

		return r;
	}

	public synchronized List getRfqsForSupplier (String username) throws StorageException
	{
		connect ();

		LinkedList rfqList = new LinkedList();

		PreparedStatement stmt = null;
		PreparedStatement stmt2 = null;

		try
		{
			stmt = conn.prepareStatement ("SELECT groupname FROM users WHERE username=?");
			stmt.setString (1, username);

			ResultSet rs = stmt.executeQuery ();
			if (!rs.next())
			{
				conn.rollback ();
				throw new StorageException ("unknown user - " + username);
			}

			String groupname = rs.getString("groupname");
			closeStatement (stmt); stmt = null;

			stmt = conn.prepareStatement (
				"SELECT rfqno,time,buyer,revno,title,preferred_supplier1,preferred_supplier2,preferred_supplier3,comment,owner,status,stamp FROM rfqs WHERE status=? AND ((preferred_supplier1='' AND preferred_supplier2='' AND preferred_supplier3='') OR (preferred_supplier1=? OR preferred_supplier2=? OR preferred_supplier3=?))");
			stmt.setString (1, Rfq.STATUS_SUBMITTED);
			stmt.setString (2, groupname);
			stmt.setString (3, groupname);
			stmt.setString (4, groupname);
			stmt.setString (1, Rfq.STATUS_SUBMITTED);

			rs = stmt.executeQuery ();

			while (rs.next())
			{
				Rfq r = new Rfq ();

				long rfqno = rs.getLong(1);

				r.setRfqno (rfqno);
				r.setTime (rs.getLong(2));
				r.setBuyer (rs.getString(3));
				r.setRevno (rs.getInt(4));
				r.setTitle (rs.getString(5));
				r.setPreferredSupplier1 (rs.getString(6));
				r.setPreferredSupplier2 (rs.getString(7));
				r.setPreferredSupplier3 (rs.getString(8));
				r.setComment (rs.getString(9));
				r.setOwner (rs.getString(10));
				r.setStatus (rs.getString(11));
				r.setStamp (rs.getLong(12));

				stmt2 = conn.prepareStatement (
					"SELECT seqno,longname,code,preferred_brand,quantity FROM rfqitems WHERE rfqno=?");
				stmt2.setLong (1, rfqno);

				LinkedList itmList = new LinkedList ();
				ResultSet rs2 = stmt2.executeQuery ();
				while (rs2.next())
				{
					RfqItem itm = new RfqItem ();

					itm.setSeqno (rs2.getInt(1));
					itm.setLongName (rs2.getString(2));
					itm.setCode (rs2.getString(3));
					itm.setPreferredBrand (rs2.getString(4));
					itm.setQuantity (rs2.getInt(5));

					itmList.addLast (itm);
				}
				
				r.setItems (itmList);
				rfqList.addLast (r);
			}

			closeStatement (stmt2); stmt2 = null;
			closeStatement (stmt); stmt = null;

			conn.commit ();
		}
		catch (SQLException e)
		{
			close ();
			throw new StorageException  (e.getMessage());
		}
		finally
		{
			closeStatement (stmt2);
			closeStatement (stmt);
		}

		return rfqList;
	}

}
