/*
 * $Id: RfqEditHandler.java,v 1.7 2007/05/19 04:43:59 bacon Exp $
 */

package com.unimar.shipneeds;

import javax.faces.model.DataModel;
import javax.faces.model.ListDataModel;

import java.util.List;
import java.util.ArrayList;
import java.util.LinkedList;
import java.util.Iterator;
import java.io.InputStream;
import java.io.IOException;

import org.apache.myfaces.custom.fileupload.UploadedFile;
import org.apache.poi.poifs.filesystem.POIFSFileSystem;
import org.apache.poi.hssf.usermodel.HSSFCell;
import org.apache.poi.hssf.usermodel.HSSFSheet;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.hssf.usermodel.HSSFRow;

public class RfqEditHandler extends Handler
{
	private RfqHandler rfqHandler;
	
	private RfqHeader currentHeader;
	private RfqItem currentItem;
	private List itemList;
	private ListDataModel itemListModel;

	private String errorMessage;
	private UploadedFile rfqFile;
	
	public RfqEditHandler ()
	{
		this.itemList = new LinkedList ();
	}
	
	public RfqHandler getRfqHandler ()
	{
		return this.rfqHandler;
	}
	
	public void setRfqHandler (RfqHandler rfqHandler)
	{
		this.rfqHandler = rfqHandler;
	}

	public DataModel getItemListModel ()
	{
		if (itemListModel == null)
			itemListModel = new ListDataModel ();
		
		itemListModel.setWrappedData (itemList);
		return itemListModel;
	}
	
	public RfqHeader getCurrentHeader ()
	{
		if (this.currentHeader == null) 
		{
			// damn dirty here!!! it could be cleaner if 
			// RfqEditHandler and RfqHandler were merged

			Rfq rfq1 = rfqHandler.getSelectedRfq ();
			if (rfq1 == null)
			{
				this.currentHeader = new RfqHeader();
			}
			else
			{
				try 
				{
					Rfq rfq2 = storage.getRfq (username, rfq1.getRfqno());
					this.currentHeader = rfq2.getHeader ();
					this.itemList = rfq2.getItems ();
					if (itemListModel != null)
						itemListModel.setWrappedData (this.itemList);
				}
				catch (StorageException e)
				{
					setErrorMessage (e.getMessage());
				}
			}
		}

		return this.currentHeader;
	}
	
	public RfqItem getCurrentItem ()
	{
		if (this.currentItem == null) 
		{
			this.currentItem = new RfqItem ();
		}

		return this.currentItem;
	}
	
	private boolean isValidItem (RfqItem item)
	{
		return item.getCode ().length () > 0 &&
		       item.getLongName ().length () > 0 &&
		       item.getQuantity () >= 0;
	}
	
	public void addItem ()
	{
		clearErrorMessage ();

		if (!isValidItem (this.currentItem)) return;
		this.itemList.add (this.currentItem);
		this.currentItem = new RfqItem ();
	}
	
	public void deleteItem ()
	{
		clearErrorMessage ();

		RfqItem selectedItem = (RfqItem)itemListModel.getRowData ();
		this.itemList.remove (selectedItem);
	}
	
	public void updateItem ()
	{
		RfqItem selectedItem = (RfqItem)itemListModel.getRowData ();
		int idx = this.itemList.indexOf (selectedItem);
		if (idx != -1)
		{
			this.itemList.set (idx, selectedItem);
		}
	}

	public String getErrorMessage ()
	{
		return this.errorMessage;
	}

	public void setErrorMessage (String m)
	{
		this.errorMessage = m;
	}

	public void clearErrorMessage ()
	{
		this.errorMessage = "";
	}

	public void clearEditData ()
	{
		this.currentItem = null;
		this.currentHeader = null;
		this.itemList.clear ();
	}

	private boolean isStatus (String s)
	{
		String status = "";

		if (this.currentHeader != null)
		{
			status = this.currentHeader.getStatus();
			if (status == null) status = "";
		}

		return status.equals(s);
	}

	public boolean isSaveRfqNewDisabled ()
	{
		return this.itemList.size () <= 0;
	}

	public boolean isSubmitRfqNewDisabled ()
	{
		return this.itemList.size () <= 0;
	}
	
	public boolean isCancelRfqNewDisabled ()
	{
		return false;
	}

	public boolean isSaveRfqEditDisabled ()
	{
		return this.itemList.size () <= 0 || 
		       !isStatus (Rfq.STATUS_DRAFT);
	}

	public boolean isSubmitRfqEditDisabled ()
	{
		return this.itemList.size () <= 0 ||
		       !isStatus (Rfq.STATUS_DRAFT);
	}

	public boolean isDropRfqEditDisabled ()
	{
		String status = "";

		if (this.currentHeader != null)
		{
			status = this.currentHeader.getStatus();
			if (status == null) status = "";
		}

		return isStatus (Rfq.STATUS_DROPPED);
	}
	
	public boolean isCancelRfqEditDisabled ()
	{
		return false;
	}

	private void saveRfq (String status) throws StorageException
	{
		Rfq rfq = new Rfq ();
		rfq.setHeader (this.currentHeader);
		rfq.setItems (this.itemList);
		
		long curTime = System.currentTimeMillis();
		rfq.setTime (curTime);
		rfq.setStamp (curTime);
		if (status != null) rfq.setStatus (status);

		storage.addRfq (username, rfq);
	}

	private void updateRfq (String status) throws StorageException
	{
		Rfq rfq = new Rfq ();
		rfq.setHeader (this.currentHeader);
		rfq.setItems (this.itemList);
		
		if (status != null) rfq.setStatus (status);

		long oldstamp = rfq.getStamp ();
		rfq.setStamp (System.currentTimeMillis());
		storage.updateRfq (username, oldstamp, rfq);
	}

	public String saveRfqNew ()
	{
		clearErrorMessage ();

		try { saveRfq (Rfq.STATUS_DRAFT); }
		catch (StorageException e)
		{
			setErrorMessage (e.getMessage());
			return "failure";
		}

		clearEditData ();
		return "success";
	}

	public String submitRfqNew ()
	{
		clearErrorMessage ();

		try { saveRfq (Rfq.STATUS_SUBMITTED); }
		catch (StorageException e)
		{
			setErrorMessage (e.getMessage());
			return "failure";
		}

		clearEditData ();
		return "success";
	}
	
	public String cancelRfqNew ()
	{
		clearErrorMessage ();
		clearEditData ();
		return "success";
	}

	public String saveRfqEdit ()
	{
		clearErrorMessage ();

		try { updateRfq (Rfq.STATUS_DRAFT); }
		catch (StorageException e)
		{
			setErrorMessage (e.getMessage());
			return "failure";
		}

		clearEditData ();
		return "success";
	}

	public String submitRfqEdit ()
	{
		clearErrorMessage ();

		try { updateRfq (Rfq.STATUS_SUBMITTED); }
		catch (StorageException e)
		{
			setErrorMessage (e.getMessage());
			return "failure";
		}

		clearEditData ();
		return "success";
	}

	public String dropRfqEdit ()
	{
		clearErrorMessage ();

		try { updateRfq (Rfq.STATUS_DROPPED); }
		catch (StorageException e)
		{
			setErrorMessage (e.getMessage());
			return "failure";
		}

		clearEditData ();
		return "success";
	}

	public String cancelRfqEdit ()
	{
		clearErrorMessage ();
		clearEditData ();
		return "success";
	}
	
	public boolean isUploadRfqNewDisabled ()
	{
		return this.itemList.size () > 0;
	}
	
	public UploadedFile getRfqFile ()
	{
		return this.rfqFile;
	}

	public void setRfqFile (UploadedFile file)
	{
		this.rfqFile = file;
	}

	public void uploadRfqNew ()
	{
		clearErrorMessage ();

		InputStream ins = null;
	       
		try
		{
			ins = rfqFile.getInputStream();
			POIFSFileSystem fs = new POIFSFileSystem (ins);
			HSSFWorkbook wb = new HSSFWorkbook (fs);
			HSSFSheet sheet = wb.getSheetAt (0);

			Iterator ri = sheet.rowIterator ();

			RfqHeader hdr = new RfqHeader ();
			LinkedList items = new LinkedList ();

			while (ri.hasNext())
			{
				HSSFRow row = (HSSFRow)ri.next ();

				Iterator ci = row.cellIterator ();
				int rn = row.getRowNum ();
				RfqItem item = new RfqItem ();

				if (rn  == 0)
				{
					while (ci.hasNext())
					{
						HSSFCell cell = (HSSFCell)ci.next();
	
						int ct = cell.getCellType ();
						int cn = cell.getCellNum ();

						if (cn == 1)
						{
							if (ct == HSSFCell.CELL_TYPE_STRING)
								hdr.setTitle (cell.getStringCellValue());
							else if (ct == HSSFCell.CELL_TYPE_NUMERIC)
								hdr.setTitle (Double.toString(cell.getNumericCellValue()));
							else if (ct != HSSFCell.CELL_TYPE_BLANK)
								hdr.setTitle ("**ERROR**");
						}
					}
				}
				else if (rn == 1)
				{
					while (ci.hasNext())
					{
						HSSFCell cell = (HSSFCell)ci.next();
	
						int ct = cell.getCellType ();
						int cn = cell.getCellNum ();

						if (cn == 1)
						{
							if (ct == HSSFCell.CELL_TYPE_STRING)
								hdr.setPreferredSupplier1 (cell.getStringCellValue());
							else if (ct == HSSFCell.CELL_TYPE_NUMERIC)
								hdr.setPreferredSupplier1 (Double.toString(cell.getNumericCellValue()));
							else if (ct != HSSFCell.CELL_TYPE_BLANK)
								hdr.setPreferredSupplier1 ("**ERROR**");
						}
					}
				}
				else if (rn == 2)
				{
					while (ci.hasNext())
					{
						HSSFCell cell = (HSSFCell)ci.next();
	
						int ct = cell.getCellType ();
						int cn = cell.getCellNum ();

						if (cn == 1)
						{
							if (ct == HSSFCell.CELL_TYPE_STRING)
								hdr.setPreferredSupplier2 (cell.getStringCellValue());
							else if (ct == HSSFCell.CELL_TYPE_NUMERIC)
								hdr.setPreferredSupplier2 (Double.toString(cell.getNumericCellValue()));
							else if (ct != HSSFCell.CELL_TYPE_BLANK)
								hdr.setPreferredSupplier2 ("**ERROR**");
						}
					}
				}
				else if (rn == 3)
				{
					while (ci.hasNext())
					{
						HSSFCell cell = (HSSFCell)ci.next();
	
						int ct = cell.getCellType ();
						int cn = cell.getCellNum ();

						if (cn == 1)
						{
							if (ct == HSSFCell.CELL_TYPE_STRING)
								hdr.setPreferredSupplier3 (cell.getStringCellValue());
							else if (ct == HSSFCell.CELL_TYPE_NUMERIC)
								hdr.setPreferredSupplier3 (Double.toString(cell.getNumericCellValue()));
							else if (ct != HSSFCell.CELL_TYPE_BLANK)
								hdr.setPreferredSupplier3 ("**ERROR**");
						}
					}
				}
				else if (rn == 4)
				{
					while (ci.hasNext())
					{
						HSSFCell cell = (HSSFCell)ci.next();
	
						int ct = cell.getCellType ();
						int cn = cell.getCellNum ();

						if (cn == 1)
						{
							if (ct == HSSFCell.CELL_TYPE_STRING)
								hdr.setComment (cell.getStringCellValue());
							else if (ct == HSSFCell.CELL_TYPE_NUMERIC)
								hdr.setComment (Double.toString(cell.getNumericCellValue()));
							else if (ct != HSSFCell.CELL_TYPE_BLANK)
								hdr.setComment ("**ERROR**");
						}
					}
				}
				else if (rn >= 7)
				{
					while (ci.hasNext())
					{
						HSSFCell cell = (HSSFCell)ci.next();
	
						int ct = cell.getCellType ();
						int cn = cell.getCellNum ();
	
						if (cn == 0)
						{
							if (ct == HSSFCell.CELL_TYPE_STRING)
								item.setLongName (cell.getStringCellValue());
							else if (ct == HSSFCell.CELL_TYPE_NUMERIC)
								item.setLongName (Double.toString(cell.getNumericCellValue()));
							else if (ct != HSSFCell.CELL_TYPE_BLANK)
								item.setLongName ("**ERROR**");
						}
						else if (cn == 1)
						{
							if (ct == HSSFCell.CELL_TYPE_STRING)
								item.setCode (cell.getStringCellValue());
							else if (ct == HSSFCell.CELL_TYPE_NUMERIC)
								item.setCode (Double.toString(cell.getNumericCellValue()));
							else if (ct != HSSFCell.CELL_TYPE_BLANK)
								item.setCode ("**ERROR**");
						}
						else if (cn == 2)
						{
							if (ct == HSSFCell.CELL_TYPE_STRING)
								item.setPreferredBrand (cell.getStringCellValue());
							else if (ct == HSSFCell.CELL_TYPE_NUMERIC)
								item.setPreferredBrand (Double.toString(cell.getNumericCellValue()));
							else if (ct != HSSFCell.CELL_TYPE_BLANK)
								item.setPreferredBrand ("**ERROR**");
						}
						else if (cn == 3)
						{
							if (ct == HSSFCell.CELL_TYPE_STRING)
							{
								int v = 0;
	
								try { Integer.parseInt (cell.getStringCellValue()); }
								catch (NumberFormatException e) {}
	
								item.setQuantity (v);
							}
							else if (ct == HSSFCell.CELL_TYPE_NUMERIC)
								item.setQuantity ((int)cell.getNumericCellValue());
							else
								item.setQuantity (0);
						}
					}

					items.addLast (item);
				}

			}

			if (items.size () > 0)
			{
				this.currentHeader = hdr;
				this.itemList = items;
				itemListModel.setWrappedData (itemList);
			}
		}
		catch (IOException e)
		{
			if (ins != null) 
			{
				try { ins.close (); } 
				catch (IOException x) {}
			}

			setErrorMessage (e.getMessage());
		}
		
	}
}
