/*
 * $Id: RfqHandler.java,v 1.6 2007/05/18 14:05:37 bacon Exp $
 */

package com.unimar.shipneeds;

import javax.faces.model.DataModel;
import javax.faces.model.ListDataModel;

import java.util.List;
import java.util.LinkedList;

public class RfqHandler extends Handler
{
	private List rfqList;
	private DataModel rfqListModel;
	private int numRowsPerPage = 10;
	private int firstRowIndex = 0;
	private String errorMessage;
	private Rfq selectedRfq;

	public RfqHandler ()
	{
		this.rfqList = new LinkedList ();
		this.rfqListModel = new ListDataModel();
		this.rfqListModel.setWrappedData (rfqList);
	}
	
	public String getErrorMessage ()
	{
		return this.errorMessage;
	}

	public void setErrorMessage (String m)
	{
		this.errorMessage = m;
	}

	public void clearErrorMessage ()
	{
		this.errorMessage = "";
	}

	public int getNumRowsPerPage ()
	{
		return this.numRowsPerPage;
	}
	
	public void setNumRowsPerPage (int n)
	{
		if (n > 0) this.numRowsPerPage = n;
	}
	
	public int getFirstRowIndex ()
	{
		return this.firstRowIndex;
	}
	
	public void setFirstRowIndex (int n)
	{
		this.firstRowIndex = n;
	}
	
	public DataModel getRfqListModel ()
	{
		clearErrorMessage ();

		try
		{
			rfqList = storage.getRfqs (username);
			// TODO: sortRFQs (rfqList);
			rfqListModel.setWrappedData (rfqList);
		}
		catch (StorageException e)
		{
			setErrorMessage (e.getMessage());
		}

		return rfqListModel;
	}
	
	public boolean isScrollFirstDisabled()
	{
		return firstRowIndex == 0;
	}
	
	public boolean isScrollLastDisabled ()
	{
		return firstRowIndex >= rfqListModel.getRowCount() - numRowsPerPage;
	}
	
	public boolean isScrollPreviousDisabled()
	{
		return firstRowIndex == 0;
	}
	
	public boolean isScrollNextDisabled()
	{
		return firstRowIndex >= rfqListModel.getRowCount() - numRowsPerPage;
	}
	
	public void scrollFirst ()
	{
		clearErrorMessage ();
		firstRowIndex = 0;
	}
	
	public void scrollLast ()
	{
		clearErrorMessage ();
		firstRowIndex = rfqListModel.getRowCount () - numRowsPerPage;
		if (firstRowIndex < 0) firstRowIndex = 0;
	}
	
	public void scrollPrevious ()
	{
		clearErrorMessage ();
		firstRowIndex -= numRowsPerPage;
		if (firstRowIndex < 0) firstRowIndex = 0;
	}
	
	public void scrollNext ()
	{
		clearErrorMessage ();
		firstRowIndex += numRowsPerPage;
		
		int rowCount = rfqListModel.getRowCount ();
		if (firstRowIndex >= rowCount)
		{
			firstRowIndex = rowCount - numRowsPerPage;
			if (firstRowIndex < 0) firstRowIndex = 0;
		}
	}

	public Rfq getSelectedRfq ()
	{
		return this.selectedRfq;
	}

	public void clearRfqSelection ()
	{
		this.selectedRfq = null;
	}

	public void selectRfq ()
	{
		clearErrorMessage ();

		Rfq rfq = (Rfq)rfqListModel.getRowData ();
		if (rfq == null) clearRfqSelection ();
		else this.selectedRfq = rfq;
	}
	
	public String newRfq ()
	{
		clearRfqSelection ();
		return "success";
	}

	public String editRfq ()
	{
		selectRfq ();

		if (selectedRfq == null) return "failure";
		String status = selectedRfq.getStatus();
		if (status == null) return "failure";

		return status.equals(Rfq.STATUS_DRAFT)? "edit": "view";
	}

	public String deleteRfq ()
	{
		return "success";
	}

}
