/*
 * $Id: UserHandler.java,v 1.5 2007/05/15 16:01:59 bacon Exp $
 */

package com.unimar.shipneeds;

import javax.faces.model.DataModel;
import javax.faces.model.ListDataModel;

import java.util.List;
import java.util.LinkedList;

public class UserHandler extends Handler
{
	private List userList;
	private ListDataModel userListModel;

	private int numRows = 10;
	private int firstRowIndex = 0;

	private User selectedUser;
	private User updatedUser = new User ();

	private String listErrorMessage;
	private String editErrorMessage;

	private GroupHandler groupHandler;

	public UserHandler ()
	{
		this.userList = new LinkedList();
		this.userListModel = new ListDataModel ();
		this.userListModel.setWrappedData (userList);
	}

	public GroupHandler getGroupHandler ()
	{
		return this.groupHandler;
	}

	public void setGroupHandler (GroupHandler groupHandler)
	{
		groupHandler.setUserHandler (this);
		this.groupHandler = groupHandler;
	}
    
	public DataModel getUserListModel ()
	{
		try 
		{ 
			reloadUserList ();
		}
		catch (StorageException e) 
		{ 
			setListErrorMessage (e.getMessage());
		}

		return this.userListModel;
	}

	public int getNumRows ()
	{
		return this.numRows;
	}

	public void setNumRows (int n)
	{
		this.numRows = n;
	}

	public int getFirstRowIndex ()
	{
		return this.firstRowIndex;
	}

	public void setFirstRowIndex (int n)
	{
		this.firstRowIndex = n;
	}

	public boolean isScrollFirstDisabled()
	{
		return firstRowIndex == 0;
	}
    
	public boolean isScrollLastDisabled ()
	{
		return firstRowIndex >= userListModel.getRowCount() - numRows;
	}
    
	public boolean isScrollPreviousDisabled()
	{
		return firstRowIndex == 0;
	}
    
	public boolean isScrollNextDisabled()
	{
		return firstRowIndex >= userListModel.getRowCount() - numRows;
	}

	public void scrollFirst ()
	{
		firstRowIndex = 0;
	}
    
	public void scrollLast ()
	{
		firstRowIndex = userListModel.getRowCount () - numRows;
		if (firstRowIndex < 0) firstRowIndex = 0;
	}
    
	public void scrollPrevious ()
	{
		firstRowIndex -= numRows;
		if (firstRowIndex < 0) firstRowIndex = 0;
	}
    
	public void scrollNext ()
	{
		firstRowIndex += numRows;
	
		int rowCount = userListModel.getRowCount ();
		if (firstRowIndex >= rowCount)
		{
			firstRowIndex = rowCount - numRows;
			if (firstRowIndex < 0) firstRowIndex = 0;
		}
	}

	public String getListErrorMessage ()
	{
		return this.listErrorMessage;
	}

	public void setListErrorMessage (String msg)
	{
		this.listErrorMessage = msg;
	}

	public String getEditErrorMessage ()
	{
		return this.editErrorMessage;
	}

	public void setEditErrorMessage (String msg)
	{
		this.editErrorMessage = msg;
	}

	public void clearErrorMessages ()
	{
		this.listErrorMessage = "";
		this.editErrorMessage = "";
	}

	public void selectUser ()
	{
		clearErrorMessages ();

		User user = (User)userListModel.getRowData ();		
		if (user == null) clearSelection ();
		else
		{
			this.selectedUser = user;
			this.updatedUser.copy (user);
		}
	}

	public User getSelectedUser ()
	{
		return this.selectedUser;
	}

	public void setSelectedUser (User g)
	{
		this.selectedUser = g;
	}

	public User getUpdatedUser ()
	{
		return this.updatedUser;
	}

	public void setUpdatedUser (User g)
	{
		this.updatedUser = g;
	}

	public void addUser ()
	{
		clearErrorMessages ();

		if (this.updatedUser.getName().length() <= 0) 
		{
			// TOOD: load message from message catalog
			setEditErrorMessage ("the user name is empty");
			return;
		}

		try 
		{ 
			if (this.groupHandler != null)
			{
				Group g = this.groupHandler.getSelectedGroup ();
				if (g != null)
				{
					this.updatedUser.setGroup (g.getName());
					this.updatedUser.setRole (g.getRole());
				}
			}

			storage.addUser (this.updatedUser);
			reloadUserList ();
			clearSelection ();
		}
		catch (StorageException e)
		{
			setEditErrorMessage (e.getMessage());
		}
	}

	public void updateUser ()
	{
		clearErrorMessages ();

		if (this.selectedUser == null ||
		    this.updatedUser.getName().length() <= 0)
		{
			setEditErrorMessage ("the user name is empty");
			return;
		}

		if (this.selectedUser.equals(this.updatedUser)) return;

		try 
		{ 
			if (this.groupHandler != null)
			{
				Group g = this.groupHandler.getSelectedGroup ();
				if (g != null)
				{
					this.updatedUser.setGroup (g.getName());
					this.updatedUser.setRole (g.getRole());
				}
			}

			storage.updateUser (
				this.selectedUser, this.updatedUser);
			reloadUserList ();
			this.selectedUser.copy (this.updatedUser);
		}
		catch (StorageException e)
		{
			setEditErrorMessage (e.getMessage());
		}
	}

	public void deleteUser ()
	{
		clearErrorMessages ();

		if (this.selectedUser == null)
		{
			setEditErrorMessage ("no user is selected");
			return;
		}

		try 
		{ 
			storage.deleteUser (this.selectedUser.getName());
			reloadUserList ();
			clearSelection ();
		}
		catch (StorageException e)
		{
			setEditErrorMessage (e.getMessage());
		}
	}

	public void clearSelection ()
	{
		this.selectedUser = null;
		this.updatedUser.clear ();
	}

	public boolean isAddUserDisabled ()
	{
		return this.selectedUser != null;
	}

	public boolean isUpdateUserDisabled ()
	{
		return this.selectedUser == null;
	}

	public boolean isDeleteUserDisabled ()
	{
		return this.selectedUser == null;
	}

	public boolean isClearSelectionDisabled ()
	{
		return this.selectedUser == null;
	}

	private void reloadUserList () throws StorageException
	{
		Group group;

		if (this.groupHandler == null || 
		    (group = this.groupHandler.getSelectedGroup()) == null) 
		{
			this.userList.clear ();
		}
		else
		{
			this.userList = storage.getUsers (group.getName()); 
			this.userListModel.setWrappedData (userList);
		}
	}

}
