/*
 * $Id: LongNumberArray.java,v 1.1 2007/07/12 12:13:50 bacon Exp $
 */

package com.unimar.shipneeds.model;

/*
 * $id$
 */

public class LongNumberArray
{	
	public final static int DEFAULT_CAPACITY = 128;
	public final static int DEFAULT_GROW_FACTOR = 128;
	public final static int INVALID_INDEX = -1;
	
	protected long[] itemBuf;
	protected int itemCount;
	protected int bufSize;
	protected int growFactor;
	
	LongNumberArray ()
	{
		this (DEFAULT_CAPACITY, DEFAULT_GROW_FACTOR);
	}

	LongNumberArray (int bufSize)
	{
		this (bufSize, DEFAULT_GROW_FACTOR);
	}

	public LongNumberArray (int bufSize, int growFactor)
	{
		if (growFactor <= 0) growFactor = 1;

		if (bufSize <= 0) 
		{
			this.itemBuf = null;
			this.bufSize = 0;
		}
		else 
		{
			this.itemBuf = new long[bufSize];
			this.bufSize = bufSize;
		}
		this.growFactor = growFactor;
		this.itemCount  = 0;
	}

	public boolean isEmpty () 
	{
		return this.itemCount == 0;
	}

	public int getSize ()
	{
		return this.itemCount;
	}

	public int getCapacity () 
	{
		return this.bufSize;
	}

	public int getGrowFactor () 
	{
		return this.growFactor;
	}

	public long[] getBuffer ()
	{
		return this.itemBuf;
	}

	public long getItem (int index)
	{
		return this.itemBuf[index];
	}

	public void setItem (int index, long value)
	{
		this.itemBuf[index] = value;
	}

	public int insertItem (int index, long value)
	{
		if (index >= this.bufSize) 
		{
			if (index - this.bufSize >= this.growFactor)
				setCapacity (index + 1);
			else 
				setCapacity (this.bufSize + this.growFactor);
		}
		else if (this.itemCount >= this.bufSize) 
		{
			setCapacity (this.bufSize + this.growFactor);
		}

		for (int i = this.itemCount; i > index; i--) 
		{
			this.itemBuf[i] = this.itemBuf[i - 1];
		}
		this.itemBuf[index] = value;
		if (index > this.itemCount) this.itemCount = index + 1;
		else this.itemCount++;

		return index;
	}
	
	public void remove (int index)
	{
		remove (index, index);
	}

	public void remove (int from_index, int to_index)
	{
		int j = from_index;
		int i = to_index + 1;
		while (i < this.itemCount) 
		{
			this.itemBuf[j++] = this.itemBuf[i++];
		}
		this.itemCount -= to_index - from_index + 1;
	}

	public int addItem (long value)
	{
		return insertItem (this.itemCount, value);
	}

	public int removeItem (long value)
	{
		int i = 0, sz = this.getSize();
		while (i < this.itemCount) 
		{
			if (value == this.itemBuf[i]) 
			{
				remove (i);
				break;
			}
			i++;
		}

		return sz - this.getSize();
	}

	public int removeItems (long value)
	{
		int i = 0, sz = this.getSize();

		while (i < this.itemCount) 
		{
			if (value == this.itemBuf[i]) remove (i);
			else i++;
		}

		return sz - this.getSize();
	}

	public void clear ()
	{
		setSize (0);
	}
	
	public void compact ()
	{
		setCapacity (this.getSize());
	}
	
	public void setSize (int size)
	{
		if (size > this.bufSize) setCapacity (size);
		this.itemCount = size;
	}
	
	public void setCapacity (int bufSize)
	{
		if (bufSize <= 0) 
		{
			this.itemBuf = new long[0];
			this.bufSize = 0;
			this.itemCount  = 0;
		}
		else 
		{
			long[] tmp = null;
			int cnt = this.itemCount;

			tmp = new long[bufSize];
			if (cnt > bufSize) cnt = bufSize;
			for (int i = 0; i < cnt; i++) 
			{
				tmp[i] = this.itemBuf[i];
			}

			this.itemBuf = tmp;
			this.bufSize = bufSize;
			this.itemCount = cnt;
		}
	}
	
	public void setGrowFactor (int growFactor)
	{
		if (growFactor <= 0) growFactor = 1;
		this.growFactor = growFactor;
	}

	public int indexOf (long value) 
	{
		for (int i = 0; i < this.itemCount; i++) 
		{
			if (this.itemBuf[i] == value) return i;
		}
		return INVALID_INDEX;
	}
	
	public int indexOf (long value, int index) 
	{
		for (int i = index; i < this.itemCount; i++) 
		{
			if (this.itemBuf[i] == value) return i;
		}
		return INVALID_INDEX;
	}
	
	public int lastIndexOf (long value)
	{
		for (int i = this.itemCount; i > 0; ) 
		{
			if (this.itemBuf[--i] == value) return i;
		}	
		return INVALID_INDEX;
	}
	
	public int lastIndexOf (long value, int index)
	{
		for (int i = index + 1; i > 0; ) 
		{
			if (this.itemBuf[--i] == value) return i;
		}
		return INVALID_INDEX;
	}
	
	public void rotate (int dir, int n)
	{
		int first, last, count, index, nk;
		long c;

		if (dir == 0) return;
		if ((n %= itemCount) == 0) return;

		if (dir > 0) n = itemCount - n;
		first = 0; nk = itemCount - n; count = 0; 

		while (count < n) 
		{
			last = first + nk;
			index = first;
			c = this.itemBuf[first];
			while (true) 
			{
				count++;
				while (index < nk) 
				{
					this.itemBuf[index] = this.itemBuf[index + n];
					index += n;
				}
				if (index == last) break;
				this.itemBuf[index] = this.itemBuf[index - nk];
				index -= nk;
			}
			this.itemBuf[last] = c; first++;
		}
	}
}