/*
 * $Id: AdminTransaction.java,v 1.12 2007/09/29 01:11:59 bacon Exp $
 */

package com.unimar.shipneeds.trans;

import com.unimar.shipneeds.model.Buyer;
import com.unimar.shipneeds.model.BuyerUser;
import com.unimar.shipneeds.model.Country;
import com.unimar.shipneeds.model.Port;
import com.unimar.shipneeds.model.Product;
import com.unimar.shipneeds.model.Supplier;
import com.unimar.shipneeds.model.SupplierCategory;
import com.unimar.shipneeds.model.SupplierUser;
import com.unimar.shipneeds.model.SystemUser;
import com.unimar.shipneeds.model.Vessel;
import com.unimar.shipneeds.util.MD5;
import com.unimar.shipneeds.view.ViewFilter;
import com.unimar.shipneeds.view.CountryViewFilter;
import com.unimar.shipneeds.view.PortViewFilter;
import com.unimar.shipneeds.view.SupplierViewFilter;
import com.unimar.shipneeds.view.SupplierCategoryViewFilter;
import com.unimar.shipneeds.view.SystemUserViewFilter;
import com.unimar.shipneeds.view.VesselViewFilter;
import com.unimar.shipneeds.view.BuyerUserViewFilter;
import com.unimar.shipneeds.view.BuyerViewFilter;
import com.unimar.shipneeds.view.ProductViewFilter;
import com.unimar.shipneeds.view.SupplierUserViewFilter;
import java.io.Serializable;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Iterator;

public class AdminTransaction extends Transaction implements Serializable
{
	public AdminTransaction ()
	{
	}
	
	public void login (String userid, String password) throws TransactionException
	{
		super.login ("system_users", userid, password);
	}
	
	//////////////////////////////////////////////
	// BEGIN COUNTRY
	//////////////////////////////////////////////
	// getCountry 
	//    returns null when id is not found
	//    throws an exception when there is an error 
	public Country getCountry (long id) throws TransactionException
	{
		Connection con = getConnection ();
		PreparedStatement stmt = null;

		try
		{
			String sql = "SELECT id,name,active,remarks FROM countries WHERE id=?";
			stmt = con.prepareStatement (sql);
			
			stmt.setLong (1, id);
			ResultSet rs = stmt.executeQuery ();
			
			if (!rs.next()) return null;
			
			Country country = new Country (
					rs.getLong (1),
					rs.getString (2),
					rs.getBoolean (3),
					rs.getString (4));
			con.commit ();			
			return country;
		}
		catch (Exception e)
		{
			rollbackTransaction (con);
			throw new TransactionException (e.getMessage (), e.getCause ());
		}
		finally
		{
			closeStatement (stmt);
			closeConnection (con);
		}
	}
	
	public List getAllCountries () throws TransactionException
	{		
		Connection con = getConnection ();
		PreparedStatement stmt = null;
		LinkedList countryList = new LinkedList ();
		
		try
		{
			stmt = con.prepareStatement (
				"SELECT id,name,active,remarks FROM countries ORDER BY name ASC");

			ResultSet rs = stmt.executeQuery ();
			
			while (rs.next ())
			{
				Country country = new Country (
					rs.getLong (1),
					rs.getString (2),
					rs.getBoolean (3),
					rs.getString (4));
				
				countryList.addLast (country);
			}
			
			con.commit ();			
		}
		catch (Exception e)
		{
			rollbackTransaction (con);
			throw new TransactionException (e.getMessage (), e.getCause ());
		}
		finally
		{
			closeStatement (stmt);
			closeConnection (con);
		}
		
		return countryList;
	}
	
	public List getCountries (int dir, Country base, ViewFilter filter) throws TransactionException
	{
		Connection con = getConnection ();
		PreparedStatement stmt = null;
		LinkedList countryList = new LinkedList ();
		
		try
		{
			StringBuffer sql = new StringBuffer("SELECT id,name,active,remarks FROM countries");
			StringBuffer whereSql = new StringBuffer();
			
			int sortKeyIndex = 0;
			int searchKeyIndex = 0;
			
			int searchKey = filter.getSearchKey ();
			String searchWord = filter.getSearchWord ();
			if (searchWord != null && searchWord.length () > 0)
			{
				if (searchKey == CountryViewFilter.SEARCH_KEY_NAME)
				{
					whereSql.append ("INSTR(name,?)>0");
					searchKeyIndex = 1;
				}
				else if (searchKey == CountryViewFilter.SEARCH_KEY_REMARKS)
				{
					whereSql.append ("INSTR(remarks,?)>0");
					searchKeyIndex = 1;
				}
			}
			
			int statusKey = filter.getStatusKey ();
			if (statusKey == CountryViewFilter.STATUS_KEY_ACTIVE)
			{
				if (whereSql.length () > 0) whereSql.append (" AND ");
				whereSql.append ("active=1");
			}
			else if (statusKey == CountryViewFilter.STATUS_KEY_INACTIVE)
			{
				if (whereSql.length () > 0) whereSql.append (" AND ");
				whereSql.append ("active=0");
			}
			
			int sortKey = filter.getSortKey ();
			if (sortKey == CountryViewFilter.SORT_KEY_ID)
			{
				if (whereSql.length () > 0) whereSql.append (" AND ");
				if (dir == GET_NEXT) whereSql.append ("id > ? ORDER BY id ASC");
				else if (dir == GET_PREVIOUS) whereSql.append ("id < ? ORDER BY id DESC");
				else whereSql.append ("id >= ? ORDER BY id ASC");
				
				sortKeyIndex = searchKeyIndex + 1;
			}
			else /* if (sortKey == CountryViewFilter.SORT_KEY_NAME) */
			{
				if (whereSql.length () > 0) whereSql.append (" AND ");
				if (dir == GET_NEXT) whereSql.append ("name > ? ORDER BY name ASC");
				else if (dir == GET_PREVIOUS) whereSql.append ("name < ? ORDER BY name DESC");
				else whereSql.append ("name >= ? ORDER BY name ASC");
				
				sortKeyIndex = searchKeyIndex + 1;
			}
			
			sql.append (" WHERE ");
			sql.append (whereSql);
			
			int limit = filter.getLimit ();
			if (limit > 0) 
			{
				sql.append (" LIMIT ");
				sql.append (limit);
			}
			
			stmt = con.prepareStatement (sql.toString ());
			
			if (sortKey == CountryViewFilter.SORT_KEY_ID) stmt.setLong (sortKeyIndex, base.getId());
			else /* if (sortKey == Country.SORT_KEY_NAME)) */ stmt.setString (sortKeyIndex, base.getName ());
			
			if (searchKeyIndex > 0) stmt.setString (searchKeyIndex, searchWord);
			
			ResultSet rs = stmt.executeQuery ();
			
			while (rs.next ())
			{
				Country country = new Country (
					rs.getLong (1),
					rs.getString (2),
					rs.getBoolean (3),
					rs.getString (4));
				
				if (dir == GET_PREVIOUS)
					countryList.addFirst (country);
				else countryList.addLast (country);
			}
			
			con.commit ();			
		}
		catch (Exception e)
		{
			rollbackTransaction (con);
			throw new TransactionException (e.getMessage (), e.getCause ());
		}
		finally
		{
			closeStatement (stmt);
			closeConnection (con);
		}
		
		return countryList;
	}
	
	public void deleteCountries (long[] ids) throws TransactionException
	{
		deleteRowsByIds ("countries", ids);
	}
	
	public void setCountryStatus (long[] ids, boolean active) throws TransactionException
	{
		setColumnActiveByIds ("countries", ids, active);
	}
	
	public void storeCountry (Country country) throws TransactionException
	{
		Connection con = getConnection ();
		PreparedStatement stmt = null;
		
		try
		{
			if (country.getId () == Country.ID_NEW)
			{
				stmt = con.prepareStatement ("INSERT INTO countries(name,active,remarks) VALUES(?,?,?)");
				stmt.setString (1, country.getName ());
				stmt.setBoolean (2, country.getActive ());
				stmt.setString (3, country.getRemarks ());
			}
			else
			{
				stmt = con.prepareStatement ("UPDATE countries SET name=?, active=?, remarks=? WHERE id=?");
				stmt.setString (1, country.getName ());
				stmt.setBoolean (2, country.getActive ());
				stmt.setString (3, country.getRemarks ());
				stmt.setLong (4, country.getId ());
			}
			
			stmt.executeUpdate ();
			
			//ResultSet rs = stmt.getGeneratedKeys ();
			con.commit ();
		}
		catch (Exception e)
		{
			rollbackTransaction (con);
			throw new TransactionException (e.getMessage (), e.getCause ());
		}
		finally
		{
			closeStatement (stmt);
			closeConnection (con);
		}
	}
	//////////////////////////////////////////////
	// END COUNTRY
	//////////////////////////////////////////////

	//////////////////////////////////////////////
	// BEGIN PORT
	//////////////////////////////////////////////
	public Port getPort (long id) throws TransactionException
	{
		Connection con = getConnection ();
		PreparedStatement stmt = null;

		try
		{
			String sql = "SELECT p.id,p.name,p.active,p.remarks,c.id,c.name,c.active,c.remarks FROM ports p, countries c WHERE p.country=c.id AND p.id=?";
			stmt = con.prepareStatement (sql);
			
			stmt.setLong (1, id);
			ResultSet rs = stmt.executeQuery ();
			
			if (!rs.next()) return null;
			
			Country country = new Country (
				rs.getLong (5),
				rs.getString (6),
				rs.getBoolean (7),
				rs.getString (8));

			Port port = new Port (
					rs.getLong (1),
					rs.getString (2),
					country,
					rs.getBoolean (3),
					rs.getString (4));
			con.commit ();			
			return port;
		}
		catch (Exception e)
		{
			rollbackTransaction (con);
			throw new TransactionException (e.getMessage (), e.getCause ());
		}
		finally
		{
			closeStatement (stmt);
			closeConnection (con);
		}
	}

	public List getAllPorts () throws TransactionException
	{
		Connection con = getConnection ();
		PreparedStatement stmt = null;
		LinkedList portList = new LinkedList ();
		
		try
		{
			stmt = con.prepareStatement (
				"SELECT p.id,p.name,p.active,p.remarks,c.id,c.name,c.active,c.remarks FROM ports p, countries c WHERE p.country=c.id ORDER BY p.name ASC");

			ResultSet rs = stmt.executeQuery ();
			
			while (rs.next ())
			{
				Country country = new Country (
					rs.getLong (5),
					rs.getString (6),
					rs.getBoolean (7),
					rs.getString (8));

				Port port = new Port (
					rs.getLong (1),
					rs.getString (2),
					country,
					rs.getBoolean (3),
					rs.getString (4));
				
				portList.addLast (port);
			}
			
			con.commit ();			
		}
		catch (Exception e)
		{
			rollbackTransaction (con);
			throw new TransactionException (e.getMessage (), e.getCause ());
		}
		finally
		{
			closeStatement (stmt);
			closeConnection (con);
		}
		
		return portList;
	}
	
		
	public List getPorts (int dir, Port base, ViewFilter filter) throws TransactionException
	{
		Connection con = getConnection ();
		PreparedStatement stmt = null;
		LinkedList portList = new LinkedList ();
		
		try
		{
			StringBuffer sql = new StringBuffer (
				"SELECT p.id,p.name,p.active,p.remarks,c.id,c.name,c.active,c.remarks FROM ports p, countries c");
			StringBuffer whereSql = new StringBuffer("p.country=c.id");
			
			int sortKeyIndex = 0;
			int searchKeyIndex = 0;
			
			int searchKey = filter.getSearchKey ();
			String searchWord = filter.getSearchWord ();
			if (searchWord != null && searchWord.length () > 0)
			{
				if (searchKey == PortViewFilter.SEARCH_KEY_NAME)
				{
					if (whereSql.length () > 0) whereSql.append (" AND ");
					whereSql.append ("INSTR(p.name,?)>0");
					searchKeyIndex = 1;
				}
				else if (searchKey == PortViewFilter.SEARCH_KEY_COUNTRY)
				{
					if (whereSql.length() > 0) whereSql.append (" AND ");
					whereSql.append ("INSTR(c.name,?)>0");
					searchKeyIndex = 1;
				}
				else if (searchKey == PortViewFilter.SEARCH_KEY_REMARKS)
				{
					if (whereSql.length () > 0) whereSql.append (" AND ");
					whereSql.append ("INSTR(p.remarks,?)>0");
					searchKeyIndex = 1;
				}
			}
			
			int statusKey = filter.getStatusKey ();
			if (statusKey == PortViewFilter.STATUS_KEY_ACTIVE)
			{
				if (whereSql.length () > 0) whereSql.append (" AND ");
				whereSql.append ("p.active=1");
			}
			else if (statusKey == PortViewFilter.STATUS_KEY_INACTIVE)
			{
				if (whereSql.length () > 0) whereSql.append (" AND ");
				whereSql.append ("p.active=0");
			}
			
			int sortKey = filter.getSortKey ();
			if (sortKey == PortViewFilter.SORT_KEY_ID)
			{
				if (whereSql.length () > 0) whereSql.append (" AND ");
				if (dir == GET_NEXT) whereSql.append ("p.id > ? ORDER BY p.id ASC");
				else if (dir == GET_PREVIOUS) whereSql.append ("p.id < ? ORDER BY p.id DESC");
				else whereSql.append ("p.id >= ? ORDER BY p.id ASC");
				
				sortKeyIndex = searchKeyIndex + 1;
			}
			else if (sortKey == PortViewFilter.SORT_KEY_NAME) 
			{
				if (whereSql.length () > 0) whereSql.append (" AND ");
				if (dir == GET_NEXT) 
					whereSql.append ("CONCAT(p.name,p.id) > CONCAT(?,?) ORDER BY CONCAT(p.name,p.id) ASC");
				else if (dir == GET_PREVIOUS) 
					whereSql.append ("CONCAT(p.name,p.id) < CONCAT(?,?) ORDER BY CONCAT(p.name,p.id) DESC");
				else whereSql.append ("CONCAT(p.name,p.id) >= CONCAT(?,?) ORDER BY CONCAT(p.name,p.id) ASC");
				
				sortKeyIndex = searchKeyIndex + 1;
			}
			else if (sortKey == PortViewFilter.SORT_KEY_COUNTRY) 
			{
				if (whereSql.length () > 0) whereSql.append (" AND ");
				if (dir == GET_NEXT) 
					whereSql.append ("CONCAT(c.name,p.id) > CONCAT(?,?) ORDER BY CONCAT(c.name,p.id) ASC");
				else if (dir == GET_PREVIOUS) 
					whereSql.append ("CONCAT(c.name,p.id) < CONCAT(?,?) ORDER BY CONCAT(c.name,p.id) DESC");
				else whereSql.append ("CONCAT(c.name,p.id) >= CONCAT(?,?) ORDER BY CONCAT(c.name,p.id) ASC");
				
				sortKeyIndex = searchKeyIndex + 1;
			}
			
			sql.append (" WHERE ");
			sql.append (whereSql);

			int limit = filter.getLimit ();
			if (limit > 0) 
			{
				sql.append (" LIMIT ");
				sql.append (limit);
			}

			stmt = con.prepareStatement (sql.toString ());
			
			if (sortKey == PortViewFilter.SORT_KEY_ID) 
			{
				stmt.setLong (sortKeyIndex, base.getId());
			}
			else if (sortKey == PortViewFilter.SORT_KEY_NAME) 
			{
				stmt.setString (sortKeyIndex, base.getName ());
				stmt.setLong (sortKeyIndex+1, base.getId ());
			}
			else if (sortKey == PortViewFilter.SORT_KEY_COUNTRY)
			{
				stmt.setString (sortKeyIndex, base.getCountry().getName ());
				stmt.setLong (sortKeyIndex+1, base.getId ());
			}
			
			if (searchKeyIndex > 0) stmt.setString (searchKeyIndex, searchWord);
			
			ResultSet rs = stmt.executeQuery ();
			
			while (rs.next ())
			{
				Country country = new Country (
					rs.getLong (5),
					rs.getString (6),
					rs.getBoolean (7),
					rs.getString (8));

				Port port = new Port (
					rs.getLong (1),
					rs.getString (2),
					country,
					rs.getBoolean (3),
					rs.getString (4));
				
				if (dir == GET_PREVIOUS)
					portList.addFirst (port);
				else portList.addLast (port);
			}
			
			con.commit ();			
		}
		catch (Exception e)
		{
			rollbackTransaction (con);
			throw new TransactionException (e.getMessage (), e.getCause ());
		}
		finally
		{
			closeStatement (stmt);
			closeConnection (con);
		}
		
		return portList;
	}
	
	public void deletePorts (long[] ids) throws TransactionException
	{
		deleteRowsByIds ("ports", ids);
	}
	
	public void setPortStatus (long[] ids, boolean active) throws TransactionException
	{
		setColumnActiveByIds ("ports", ids, active);
	}
	
	public void storePort (Port port) throws TransactionException
	{
		Connection con = getConnection ();
		PreparedStatement stmt = null;
		
		try
		{
			Country country = port.getCountry();

			if (country.getId () == Country.ID_NEW)
			{
				// load the country by the name
				stmt = con.prepareStatement ("SELECT id FROM countries WHERE name=?");
				stmt.setString (1, country.getName());

				ResultSet rs = stmt.executeQuery ();
				if (rs.next())
				{
					country.setId (rs.getLong (1));
				}
				/* if it is not resolved, the update sql below would trigger an exception 
				 * TODO: do it need to handle this case???
				else
				{
				}
				*/

				closeStatement (stmt); stmt = null;
			}

			if (port.getId () == Port.ID_NEW)
			{
				stmt = con.prepareStatement ("INSERT INTO ports(name,country,active,remarks) VALUES(?,?,?,?)");
				stmt.setString (1, port.getName ());
				stmt.setLong (2, country.getId ());
				stmt.setBoolean (3, port.getActive ());
				stmt.setString (4, port.getRemarks ());
			}
			else
			{
				stmt = con.prepareStatement ("UPDATE ports SET name=?, country=?, active=?, remarks=? WHERE id=?");
				stmt.setString (1, port.getName ());
				stmt.setLong (2, country.getId ());
				stmt.setBoolean (3, port.getActive ());
				stmt.setString (4, port.getRemarks ());
				stmt.setLong (5, port.getId ());
			}
			
			stmt.executeUpdate ();
			
			//ResultSet rs = stmt.getGeneratedKeys ();
			con.commit ();
		}
		catch (Exception e)
		{
			rollbackTransaction (con);
			throw new TransactionException (e.getMessage (), e.getCause ());
		}
		finally
		{
			closeStatement (stmt);
			closeConnection (con);
		}
	}
	//////////////////////////////////////////////
	// END PORT
	//////////////////////////////////////////////
	
	//////////////////////////////////////////////
	// BEGIN SUPPLIER_CATEGORY
	//////////////////////////////////////////////
	// getSupplierCategory 
	//    returns null when id is not found
	//    throws an exception when there is an error 
	public SupplierCategory getSupplierCategory (long id) throws TransactionException
	{
		Connection con = getConnection ();
		PreparedStatement stmt = null;

		try
		{
			String sql = "SELECT id,name,active,remarks FROM supplier_categories WHERE id=?";
			stmt = con.prepareStatement (sql);
			
			stmt.setLong (1, id);
			ResultSet rs = stmt.executeQuery ();
			
			if (!rs.next()) return null;
			
			SupplierCategory supplierCategory = new SupplierCategory (
					rs.getLong (1),
					rs.getString (2),
					rs.getBoolean (3),
					rs.getString (4));
			con.commit ();			
			return supplierCategory;
		}
		catch (Exception e)
		{
			rollbackTransaction (con);
			throw new TransactionException (e.getMessage (), e.getCause ());
		}
		finally
		{
			closeStatement (stmt);
			closeConnection (con);
		}
	}
	
	public List getAllSupplierCategories () throws TransactionException
	{		
		Connection con = getConnection ();
		PreparedStatement stmt = null;
		LinkedList supplierCategoryList = new LinkedList ();
		
		try
		{
			stmt = con.prepareStatement (
				"SELECT id,name,active,remarks FROM supplier_categories ORDER BY name ASC");

			ResultSet rs = stmt.executeQuery ();
			
			while (rs.next ())
			{
				SupplierCategory supplierCategory = new SupplierCategory (
					rs.getLong (1),
					rs.getString (2),
					rs.getBoolean (3),
					rs.getString (4));
				
				supplierCategoryList.addLast (supplierCategory);
			}
			
			con.commit ();			
		}
		catch (Exception e)
		{
			rollbackTransaction (con);
			throw new TransactionException (e.getMessage (), e.getCause ());
		}
		finally
		{
			closeStatement (stmt);
			closeConnection (con);
		}
		
		return supplierCategoryList;
	}
	
	public List getSupplierCategories (int dir, SupplierCategory base, ViewFilter filter) throws TransactionException
	{
		Connection con = getConnection ();
		PreparedStatement stmt = null;
		LinkedList supplierCategoryList = new LinkedList ();
		
		try
		{
			StringBuffer sql = new StringBuffer("SELECT id,name,active,remarks FROM supplier_categories");
			StringBuffer whereSql = new StringBuffer();
			
			int sortKeyIndex = 0;
			int searchKeyIndex = 0;
			
			int searchKey = filter.getSearchKey ();
			String searchWord = filter.getSearchWord ();
			if (searchWord != null && searchWord.length () > 0)
			{
				if (searchKey == SupplierCategoryViewFilter.SEARCH_KEY_NAME)
				{
					whereSql.append ("INSTR(name,?)>0");
					searchKeyIndex = 1;
				}
				else if (searchKey == SupplierCategoryViewFilter.SEARCH_KEY_REMARKS)
				{
					whereSql.append ("INSTR(remarks,?)>0");
					searchKeyIndex = 1;
				}
			}
			
			int statusKey = filter.getStatusKey ();
			if (statusKey == SupplierCategoryViewFilter.STATUS_KEY_ACTIVE)
			{
				if (whereSql.length () > 0) whereSql.append (" AND ");
				whereSql.append ("active=1");
			}
			else if (statusKey == SupplierCategoryViewFilter.STATUS_KEY_INACTIVE)
			{
				if (whereSql.length () > 0) whereSql.append (" AND ");
				whereSql.append ("active=0");
			}
			
			int sortKey = filter.getSortKey ();
			if (sortKey == SupplierCategoryViewFilter.SORT_KEY_ID)
			{
				if (whereSql.length () > 0) whereSql.append (" AND ");
				if (dir == GET_NEXT) whereSql.append ("id > ? ORDER BY id ASC");
				else if (dir == GET_PREVIOUS) whereSql.append ("id < ? ORDER BY id DESC");
				else whereSql.append ("id >= ? ORDER BY id ASC");
				
				sortKeyIndex = searchKeyIndex + 1;
			}
			else /* if (sortKey == SupplierCategoryViewFilter.SORT_KEY_NAME) */
			{
				if (whereSql.length () > 0) whereSql.append (" AND ");
				if (dir == GET_NEXT) whereSql.append ("name > ? ORDER BY name ASC");
				else if (dir == GET_PREVIOUS) whereSql.append ("name < ? ORDER BY name DESC");
				else whereSql.append ("name >= ? ORDER BY name ASC");
				
				sortKeyIndex = searchKeyIndex + 1;
			}
			
			sql.append (" WHERE ");
			sql.append (whereSql);
			
			int limit = filter.getLimit ();
			if (limit > 0) 
			{
				sql.append (" LIMIT ");
				sql.append (limit);
			}
			
			stmt = con.prepareStatement (sql.toString ());
			
			if (sortKey == SupplierCategoryViewFilter.SORT_KEY_ID) stmt.setLong (sortKeyIndex, base.getId());
			else /* if (sortKey == SupplierCategory.SORT_KEY_NAME)) */ stmt.setString (sortKeyIndex, base.getName ());
			
			if (searchKeyIndex > 0) stmt.setString (searchKeyIndex, searchWord);
			
			ResultSet rs = stmt.executeQuery ();
			
			while (rs.next ())
			{
				SupplierCategory supplierCategory = new SupplierCategory (
					rs.getLong (1),
					rs.getString (2),
					rs.getBoolean (3),
					rs.getString (4));
				
				if (dir == GET_PREVIOUS)
					supplierCategoryList.addFirst (supplierCategory);
				else supplierCategoryList.addLast (supplierCategory);
			}
			
			con.commit ();			
		}
		catch (Exception e)
		{
			rollbackTransaction (con);
			throw new TransactionException (e.getMessage (), e.getCause ());
		}
		finally
		{
			closeStatement (stmt);
			closeConnection (con);
		}
		
		return supplierCategoryList;
	}
	
	public void deleteSupplierCategories (long[] ids) throws TransactionException
	{
		deleteRowsByIds ("supplier_categories", ids);
	}
	
	public void setSupplierCategoryStatus (long[] ids, boolean active) throws TransactionException
	{
		setColumnActiveByIds ("supplier_categories", ids, active);
	}
	
	public void storeSupplierCategory (SupplierCategory supplierCategory) throws TransactionException
	{
		Connection con = getConnection ();
		PreparedStatement stmt = null;
		
		try
		{
			if (supplierCategory.getId () == SupplierCategory.ID_NEW)
			{
				stmt = con.prepareStatement ("INSERT INTO supplier_categories(name,active,remarks) VALUES(?,?,?)");
				stmt.setString (1, supplierCategory.getName ());
				stmt.setBoolean (2, supplierCategory.getActive ());
				stmt.setString (3, supplierCategory.getRemarks ());
			}
			else
			{
				stmt = con.prepareStatement ("UPDATE supplier_categories SET name=?, active=?, remarks=? WHERE id=?");
				stmt.setString (1, supplierCategory.getName ());
				stmt.setBoolean (2, supplierCategory.getActive ());
				stmt.setString (3, supplierCategory.getRemarks ());
				stmt.setLong (4, supplierCategory.getId ());
			}
			
			stmt.executeUpdate ();
			
			//ResultSet rs = stmt.getGeneratedKeys ();
			con.commit ();
		}
		catch (Exception e)
		{
			rollbackTransaction (con);
			throw new TransactionException (e.getMessage (), e.getCause ());
		}
		finally
		{
			closeStatement (stmt);
			closeConnection (con);
		}
	}
	//////////////////////////////////////////////
	// END SUPPLIER_CATEGORY
	//////////////////////////////////////////////

	//////////////////////////////////////////////
	// BEGIN SYSTEM USER
	//////////////////////////////////////////////
	// getSystemUser 
	//    returns null when id is not found
	//    throws an exception when there is an error 
	public SystemUser getSystemUser (long id) throws TransactionException
	{
		Connection con = getConnection ();
		PreparedStatement stmt = null;

		try
		{
			String sql = "SELECT id,userid,name,email,active,remarks FROM system_users WHERE id=?";
			stmt = con.prepareStatement (sql);
			
			stmt.setLong (1, id);
			ResultSet rs = stmt.executeQuery ();
			
			if (!rs.next()) return null;
			
			SystemUser systemUser = new SystemUser (
					rs.getLong (1),
					rs.getString (2),
					"", // password
					rs.getString (3),
					rs.getString (4),
					rs.getBoolean (5),
					rs.getString (6));
			con.commit ();			
			return systemUser;
		}
		catch (Exception e)
		{
			rollbackTransaction (con);
			throw new TransactionException (e.getMessage (), e.getCause ());
		}
		finally
		{
			closeStatement (stmt);
			closeConnection (con);
		}
	}
	
	public List getSystemUsers (int dir, SystemUser base, ViewFilter filter) throws TransactionException
	{
		Connection con = getConnection ();
		PreparedStatement stmt = null;
		LinkedList systemUserList = new LinkedList ();
		
		try
		{
			StringBuffer sql = new StringBuffer("SELECT id,userid,name,email,active,remarks FROM system_users");
			StringBuffer whereSql = new StringBuffer();
			
			int sortKeyIndex = 0;
			int searchKeyIndex = 0;
			
			int searchKey = filter.getSearchKey ();
			String searchWord = filter.getSearchWord ();
			if (searchWord != null && searchWord.length () > 0)
			{
				if (searchKey == SystemUserViewFilter.SEARCH_KEY_USERID)
				{
					whereSql.append ("INSTR(userid,?)>0");
					searchKeyIndex = 1;
				}
				else if (searchKey == SystemUserViewFilter.SEARCH_KEY_NAME)
				{
					whereSql.append ("INSTR(name,?)>0");
					searchKeyIndex = 1;
				}
				else if (searchKey == SystemUserViewFilter.SEARCH_KEY_EMAIL)
				{
					whereSql.append ("INSTR(email,?)>0");
					searchKeyIndex = 1;
				}
				else if (searchKey == SystemUserViewFilter.SEARCH_KEY_REMARKS)
				{
					whereSql.append ("INSTR(remarks,?)>0");
					searchKeyIndex = 1;
				}
			}
			
			int statusKey = filter.getStatusKey ();
			if (statusKey == SystemUserViewFilter.STATUS_KEY_ACTIVE)
			{
				if (whereSql.length () > 0) whereSql.append (" AND ");
				whereSql.append ("active=1");
			}
			else if (statusKey == SystemUserViewFilter.STATUS_KEY_INACTIVE)
			{
				if (whereSql.length () > 0) whereSql.append (" AND ");
				whereSql.append ("active=0");
			}
			
			int sortKey = filter.getSortKey ();
			if (sortKey == SystemUserViewFilter.SORT_KEY_ID)
			{
				if (whereSql.length () > 0) whereSql.append (" AND ");
				if (dir == GET_NEXT) whereSql.append ("id > ? ORDER BY id ASC");
				else if (dir == GET_PREVIOUS) whereSql.append ("id < ? ORDER BY id DESC");
				else whereSql.append ("id >= ? ORDER BY id ASC");
				
				sortKeyIndex = searchKeyIndex + 1;
			}
			else if (sortKey == SystemUserViewFilter.SORT_KEY_USERID)
			{
				if (whereSql.length () > 0) whereSql.append (" AND ");
				if (dir == GET_NEXT) whereSql.append ("userid > ? ORDER BY userid ASC");
				else if (dir == GET_PREVIOUS) whereSql.append ("userid < ? ORDER BY userid DESC");
				else whereSql.append ("userid >= ? ORDER BY userid ASC");
				
				sortKeyIndex = searchKeyIndex + 1;
			}
			else if (sortKey == SystemUserViewFilter.SORT_KEY_NAME) 
			{
				if (whereSql.length () > 0) whereSql.append (" AND ");
				if (dir == GET_NEXT) 
					whereSql.append ("(CONCAT(name,id) > CONCAT(?,?)) ORDER BY CONCAT(name,id) ASC");
				else if (dir == GET_PREVIOUS) 
					whereSql.append ("(CONCAT(name,id) < CONCAT(?,?)) ORDER BY CONCAT(name,id) DESC");
				else whereSql.append ("(CONCAT(name,id) >= CONCAT(?,?)) ORDER BY CONCAT(name,id) ASC");
				
				sortKeyIndex = searchKeyIndex + 1;
			}
			else /* if (sortKey == SystemUserViewFilter.SORT_KEY_EMAIL) */
			{
				if (whereSql.length () > 0) whereSql.append (" AND ");
				if (dir == GET_NEXT) whereSql.append ("email > ? ORDER BY email ASC");
				else if (dir == GET_PREVIOUS) whereSql.append ("email < ? ORDER BY email DESC");
				else whereSql.append ("email >= ? ORDER BY email ASC");
				
				sortKeyIndex = searchKeyIndex + 1;
			}
			
			sql.append (" WHERE ");
			sql.append (whereSql);

			int limit = filter.getLimit ();
			if (limit > 0) 
			{
				sql.append (" LIMIT ");
				sql.append (limit);
			}
			
			stmt = con.prepareStatement (sql.toString ());
			
			if (sortKey == SystemUserViewFilter.SORT_KEY_ID) 
			{
				stmt.setLong (sortKeyIndex, base.getId());
			}
			else if (sortKey == SystemUserViewFilter.SORT_KEY_USERID)  
			{
				stmt.setString (sortKeyIndex, base.getUserid ());
			}
			else if (sortKey == SystemUserViewFilter.SORT_KEY_NAME)  
			{
				stmt.setString (sortKeyIndex, base.getName ());
				stmt.setLong (sortKeyIndex+1, base.getId ());
			}
			else /*(sortKey == SystemUserViewFilter.SORT_KEY_EMAIL)*/  
			{
				stmt.setString (sortKeyIndex, base.getName ());
			}
			
			if (searchKeyIndex > 0) stmt.setString (searchKeyIndex, searchWord);
			
			ResultSet rs = stmt.executeQuery ();
			
			while (rs.next ())
			{
				SystemUser systemUser = new SystemUser (
					rs.getLong (1),
					rs.getString (2),
					"", // password
					rs.getString (3),
					rs.getString (4),
					rs.getBoolean (5),
					rs.getString (6));
				
				if (dir == GET_PREVIOUS)
					systemUserList.addFirst (systemUser);
				else systemUserList.addLast (systemUser);
			}
			
			con.commit ();			
		}
		catch (Exception e)
		{
			rollbackTransaction (con);
			throw new TransactionException (e.getMessage (), e.getCause ());
		}
		finally
		{
			closeStatement (stmt);
			closeConnection (con);
		}
		
		return systemUserList;
	}
	
	public void deleteSystemUsers (long[] ids) throws TransactionException
	{
		deleteRowsByIds ("system_users", ids);
	}
	
	public void setSystemUserStatus (
		long[] ids, boolean active) throws TransactionException
	{
		setColumnActiveByIds ("system_users", ids, active);
	}
	
	public void storeSystemUser (SystemUser systemUser) throws TransactionException
	{
		Connection con = getConnection ();
		PreparedStatement stmt = null;
		
		try
		{
			if (systemUser.getId () == SystemUser.ID_NEW)
			{
				MD5 md5 = new MD5();
				String pwd = systemUser.getUserid () + systemUser.getPassword ();
				String hashedPwd = md5.hashData (pwd.getBytes ("utf-8"));
				
				stmt = con.prepareStatement ("INSERT INTO system_users(userid,password,name,email,active,remarks) VALUES(?,?,?,?,?,?)");
				stmt.setString (1, systemUser.getUserid ());
				stmt.setString (2, hashedPwd);
				stmt.setString (3, systemUser.getName ());
				stmt.setString (4, systemUser.getEmail ());
				stmt.setBoolean (5, systemUser.getActive ());
				stmt.setString (6, systemUser.getRemarks ());
			}
			else
			{
				String passwd = systemUser.getPassword();
				String sql;

				if (passwd == null)
				{
					sql = "UPDATE system_users SET userid=?, name=?, email=?, active=?, remarks=? WHERE id=?";
				}
				else
				{
					sql = "UPDATE system_users SET userid=?, password=?, name=?, email=?, active=?, remarks=? WHERE id=?";
				}

				stmt = con.prepareStatement (sql);

				int fid = 1;

				stmt.setString (fid++, systemUser.getUserid ());
				if (passwd != null) 
				{
					MD5 md5 = new MD5();
					String hashedPwd = md5.hashData ((systemUser.getUserid () + passwd).getBytes ("utf-8"));
					stmt.setString (fid++, hashedPwd);
				}
				stmt.setString (fid++, systemUser.getName ());
				stmt.setString (fid++, systemUser.getEmail ());
				stmt.setBoolean (fid++, systemUser.getActive ());
				stmt.setString (fid++, systemUser.getRemarks ());
				stmt.setLong (fid++, systemUser.getId ());
			}
			
			stmt.executeUpdate ();
			
			//ResultSet rs = stmt.getGeneratedKeys ();
			con.commit ();
		}
		catch (Exception e)
		{
			rollbackTransaction (con);			
			throw new TransactionException (e.getMessage (), e.getCause ());
		}
		finally
		{			
			closeStatement (stmt);
			closeConnection (con);
		}
	}
	// END SYSTEM USER

	//////////////////////////////////////////////
	// BEGIN VESSEL
	//////////////////////////////////////////////
	// getVessel 
	//    returns null when id is not found
	//    throws an exception when there is an error 
	public Vessel getVessel (long id) throws TransactionException
	{
		Connection con = getConnection ();
		PreparedStatement stmt = null;

		try
		{
			String sql = "SELECT v.id,v.name,v.callsign,v.owner,v.active,v.remarks,c1.id,c1.name,c1.active,c1.remarks,b.id,b.name,b.contact,b.department,b.address,b.zipcode,b.email,b.telno,b.faxno,b.telex,b.active,b.remarks,c2.id,c2.name,c2.active,c2.remarks FROM vessels v, buyers b, countries c1, countries c2 WHERE v.country=c1.id AND v.buyer=b.id AND b.country=c2.id AND v.id=?";
			stmt = con.prepareStatement (sql);
			
			stmt.setLong (1, id);
			ResultSet rs = stmt.executeQuery ();
			
			if (!rs.next()) return null;
			
			Country c2 = new Country (
				rs.getLong (23),
				rs.getString (24),
				rs.getBoolean (25),
				rs.getString (26));

			Buyer buyer = new Buyer (
				rs.getLong (11),     // id
				rs.getString (12),   // name
				rs.getString (13),   // contact
				rs.getString (14),   // department
				rs.getString (15),   // address
				rs.getString (16),   // zipcode
				c2,
				rs.getString (17),   // email
				rs.getString (18),   // telno
				rs.getString (19),   // faxno
				rs.getString (20),  // telex
				rs.getBoolean (21), // active
				rs.getString (22)); // remarks

			Country c1 = new Country (
				rs.getLong (7),
				rs.getString (8),
				rs.getBoolean (9),
				rs.getString (10));

			Vessel vessel = new Vessel (
				rs.getLong (1),
				rs.getString (2),
				buyer,
				c1,
				rs.getString (3), // callsign
				rs.getString (4), // owner
				rs.getBoolean (5), // active
				rs.getString (6)); // remarks

			con.commit ();			
			return vessel;
		}
		catch (Exception e)
		{
			rollbackTransaction (con);
			throw new TransactionException (e.getMessage (), e.getCause ());
		}
		finally
		{
			closeStatement (stmt);
			closeConnection (con);
		}
	}
	
	
	public List getVessels (int dir, Vessel base, ViewFilter filter) throws TransactionException
	{
		Connection con = getConnection ();
		PreparedStatement stmt = null;
		LinkedList vesselList = new LinkedList ();
		
		try
		{
			StringBuffer sql = new StringBuffer("SELECT v.id,v.name,v.callsign,v.owner,v.active,v.remarks,c1.id,c1.name,c1.active,c1.remarks,b.id,b.name,b.contact,b.department,b.address,b.zipcode,b.email,b.telno,b.faxno,b.telex,b.active,b.remarks,c2.id,c2.name,c2.active,c2.remarks FROM vessels v, buyers b, countries c1, countries c2");
			StringBuffer whereSql = new StringBuffer("v.country=c1.id AND v.buyer=b.id AND b.country=c2.id");
			
			int sortKeyIndex = 0;
			int searchKeyIndex = 0;
			
			int searchKey = filter.getSearchKey ();
			String searchWord = filter.getSearchWord ();
			if (searchWord != null && searchWord.length () > 0)
			{
				if (searchKey == VesselViewFilter.SEARCH_KEY_NAME)
				{
					if (whereSql.length () > 0) whereSql.append (" AND ");
					whereSql.append ("INSTR(v.name,?)>0");
					searchKeyIndex = 1;
				}
				else if (searchKey == VesselViewFilter.SEARCH_KEY_BUYER)
				{
					if (whereSql.length () > 0) whereSql.append (" AND ");
					whereSql.append ("INSTR(b.name,?)>0");
					searchKeyIndex = 1;
				}
				else if (searchKey == VesselViewFilter.SEARCH_KEY_COUNTRY)
				{
					if (whereSql.length () > 0) whereSql.append (" AND ");
					whereSql.append ("INSTR(c1.name,?)>0");
					searchKeyIndex = 1;
				}
				else if (searchKey == VesselViewFilter.SEARCH_KEY_CALLSIGN)
				{
					if (whereSql.length () > 0) whereSql.append (" AND ");
					whereSql.append ("INSTR(v.callsign,?)>0");
					searchKeyIndex = 1;
				}
				else if (searchKey == VesselViewFilter.SEARCH_KEY_OWNER)
				{
					if (whereSql.length () > 0) whereSql.append (" AND ");
					whereSql.append ("INSTR(v.owner,?)>0");
					searchKeyIndex = 1;
				}
				else if (searchKey == VesselViewFilter.SEARCH_KEY_REMARKS)
				{
					if (whereSql.length () > 0) whereSql.append (" AND ");
					whereSql.append ("INSTR(v.remarks,?)>0");
					searchKeyIndex = 1;
				}
			}
			
			int statusKey = filter.getStatusKey ();
			if (statusKey == VesselViewFilter.STATUS_KEY_ACTIVE)
			{
				if (whereSql.length () > 0) whereSql.append (" AND ");
				whereSql.append ("v.active=1");
			}
			else if (statusKey == VesselViewFilter.STATUS_KEY_INACTIVE)
			{
				if (whereSql.length () > 0) whereSql.append (" AND ");
				whereSql.append ("v.active=0");
			}
			
			int sortKey = filter.getSortKey ();
			if (sortKey == VesselViewFilter.SORT_KEY_ID)
			{
				if (whereSql.length () > 0) whereSql.append (" AND ");
				if (dir == GET_NEXT) whereSql.append ("v.id > ? ORDER BY v.id ASC");
				else if (dir == GET_PREVIOUS) whereSql.append ("v.id < ? ORDER BY v.id DESC");
				else whereSql.append ("v.id >= ? ORDER BY v.id ASC");
				
				sortKeyIndex = searchKeyIndex + 1;
			}
			else if (sortKey == VesselViewFilter.SORT_KEY_NAME) 
			{
				if (whereSql.length () > 0) whereSql.append (" AND ");
				if (dir == GET_NEXT) whereSql.append ("v.name > ? ORDER BY v.name ASC");
				else if (dir == GET_PREVIOUS) whereSql.append ("v.name < ? ORDER BY v.name DESC");
				else whereSql.append ("v.name >= ? ORDER BY v.name ASC");
				
				sortKeyIndex = searchKeyIndex + 1;
			}
			else if (sortKey == VesselViewFilter.SORT_KEY_BUYER) 
			{
				if (whereSql.length () > 0) whereSql.append (" AND ");
				if (dir == GET_NEXT) 
					whereSql.append ("CONCAT(b.name,v.id) > CONCAT(?,?) ORDER BY CONCAT(b.name,v.id) ASC");
				else if (dir == GET_PREVIOUS) 
					whereSql.append ("CONCAT(b.name,v.id) < CONCAT(?,?) ORDER BY CONCAT(b.name,v.id) DESC");
				else whereSql.append ("CONCAT(b.name,v.id) >= CONCAT(?,?) ORDER BY CONCAT(b.name,v.id) ASC");

				sortKeyIndex = searchKeyIndex + 1;
			}
			else if (sortKey == VesselViewFilter.SORT_KEY_COUNTRY) 
			{
				if (whereSql.length () > 0) whereSql.append (" AND ");
				if (dir == GET_NEXT) 
					whereSql.append ("CONCAT(c1.name,v.id) > CONCAT(?,?) ORDER BY CONCAT(c1.name,v.id) ASC");
				else if (dir == GET_PREVIOUS) 
					whereSql.append ("CONCAT(c1.name,v.id) < CONCAT(?,?) ORDER BY CONCAT(c1.name,v.id) DESC");
				else whereSql.append ("CONCAT(c1.name,v.id) >= CONCAT(?,?) ORDER BY CONCAT(c1.name,v.id) ASC");

				sortKeyIndex = searchKeyIndex + 1;
			}
			else if (sortKey == VesselViewFilter.SORT_KEY_CALLSIGN) 
			{
				if (whereSql.length () > 0) whereSql.append (" AND ");
				if (dir == GET_NEXT) whereSql.append ("v.callsign > ? ORDER BY v.callsign ASC");
				else if (dir == GET_PREVIOUS) whereSql.append ("v.callsign < ? ORDER BY v.callsign DESC");
				else whereSql.append ("v.callsign >= ? ORDER BY v.callsign ASC");
				
				sortKeyIndex = searchKeyIndex + 1;
			}
			
			sql.append (" WHERE ");
			sql.append (whereSql);
			
			int limit = filter.getLimit ();
			if (limit > 0) 
			{
				sql.append (" LIMIT ");
				sql.append (limit);
			}
			
			stmt = con.prepareStatement (sql.toString ());
			
			if (sortKey == VesselViewFilter.SORT_KEY_ID) 
			{
				stmt.setLong (sortKeyIndex, base.getId());
			}
			else if (sortKey == VesselViewFilter.SORT_KEY_NAME) 
			{
				stmt.setString (sortKeyIndex, base.getName());
			}
			else if (sortKey == VesselViewFilter.SORT_KEY_BUYER)
			{
				stmt.setString (sortKeyIndex, base.getBuyer().getName ());
				stmt.setLong (sortKeyIndex+1, base.getId ());
			}
			else if (sortKey == VesselViewFilter.SORT_KEY_COUNTRY)
			{
				stmt.setString (sortKeyIndex, base.getCountry().getName ());
				stmt.setLong (sortKeyIndex+1, base.getId ());
			}
			else if (sortKey == VesselViewFilter.SORT_KEY_CALLSIGN) 
			{
				stmt.setString (sortKeyIndex, base.getCallSign());
			}
			
			if (searchKeyIndex > 0) stmt.setString (searchKeyIndex, searchWord);
			
			ResultSet rs = stmt.executeQuery ();
			
			while (rs.next ())
			{
				Country c2 = new Country (
					rs.getLong (23),
					rs.getString (24),
					rs.getBoolean (25),
					rs.getString (26));

				Buyer buyer = new Buyer (
					rs.getLong (11),     // id
					rs.getString (12),   // name
					rs.getString (13),   // contact
					rs.getString (14),   // department
					rs.getString (15),   // address
					rs.getString (16),   // zipcode
					c2,
					rs.getString (17),   // email
					rs.getString (18),   // telno
					rs.getString (19),   // faxno
					rs.getString (20),  // telex
					rs.getBoolean (21), // active
					rs.getString (22)); // remarks

				Country c1 = new Country (
					rs.getLong (7),
					rs.getString (8),
					rs.getBoolean (9),
					rs.getString (10));

				Vessel vessel = new Vessel (
					rs.getLong (1),
					rs.getString (2),
					buyer,
					c1,
					rs.getString (3), // callsign
					rs.getString(4), // owner
					rs.getBoolean (5), // active
					rs.getString (6)); // remarks
				
				if (dir == GET_PREVIOUS)
					vesselList.addFirst (vessel);
				else vesselList.addLast (vessel);
			}

			con.commit ();			
		}
		catch (Exception e)
		{
			rollbackTransaction (con);
			throw new TransactionException (e.getMessage (), e.getCause ());
		}
		finally
		{
			closeStatement (stmt);
			closeConnection (con);
		}
		
		return vesselList;
	}
	
	public void deleteVessels (long[] ids) throws TransactionException
	{
		deleteRowsByIds ("vessels", ids);
	}
	
	public void setVesselStatus (long[] ids, boolean active) throws TransactionException
	{
		setColumnActiveByIds ("vessels", ids, active);
	}
	
	public void storeVessel (Vessel vessel) throws TransactionException
	{
		Connection con = getConnection ();
		PreparedStatement stmt = null;
		
		try
		{
			Buyer buyer = vessel.getBuyer ();
			if (buyer.getId () == Buyer.ID_NEW)
			{
				// load a buyer by the name
				stmt = con.prepareStatement ("SELECT id FROM buyers WHERE name=?");
				stmt.setString (1, buyer.getName());

				ResultSet rs = stmt.executeQuery ();
				if (rs.next())
				{
					buyer.setId (rs.getLong (1));
				}
				closeStatement (stmt); stmt = null;
			}

			Country country = vessel.getCountry ();

			if (country.getId () == Country.ID_NEW)
			{
				// load a country by the name
				stmt = con.prepareStatement ("SELECT id FROM countries WHERE name=?");
				stmt.setString (1, country.getName());

				ResultSet rs = stmt.executeQuery ();
				if (rs.next())
				{
					country.setId (rs.getLong (1));
				}
				closeStatement (stmt); stmt = null;
			}


			if (vessel.getId () == Vessel.ID_NEW)
			{
				stmt = con.prepareStatement ("INSERT INTO vessels(name,buyer,country,callsign,owner,active,remarks) VALUES(?,?,?,?,?,?,?)");
				stmt.setString (1, vessel.getName ());
				stmt.setLong (2, buyer.getId());
				stmt.setLong(3, country.getId());
				stmt.setString (4, vessel.getCallSign());
				stmt.setString (5, vessel.getOwner());
				stmt.setBoolean (6, vessel.getActive ());
				stmt.setString (7, vessel.getRemarks ());
			}
			else
			{
				stmt = con.prepareStatement ("UPDATE vessels SET name=?, buyer=?, country=?, callsign=?, owner=?, active=?, remarks=? WHERE id=?");
				stmt.setString (1, vessel.getName ());
				stmt.setString (1, vessel.getName ());
				stmt.setLong (2, buyer.getId());
				stmt.setLong(3, country.getId());
				stmt.setString (4, vessel.getCallSign());
				stmt.setString (5, vessel.getOwner());
				stmt.setBoolean (6, vessel.getActive ());
				stmt.setString (7, vessel.getRemarks ());
				stmt.setLong (8, vessel.getId ());
			}
			
			stmt.executeUpdate ();
			
			//ResultSet rs = stmt.getGeneratedKeys ();
			con.commit ();
		}
		catch (Exception e)
		{
			rollbackTransaction (con);
			throw new TransactionException (e.getMessage (), e.getCause ());
		}
		finally
		{
			closeStatement (stmt);
			closeConnection (con);
		}
	}
	//////////////////////////////////////////////
	// END VESSEL
	//////////////////////////////////////////////
	
	//////////////////////////////////////////////
	// BEGIN SUPPLIER
	//////////////////////////////////////////////
	public Supplier getSupplier (long id) throws TransactionException
	{
		Connection con = getConnection ();
		PreparedStatement stmt = null;

		try
		{
			String sql = "SELECT s.id,s.name,s.contact,s.department,s.address,s.zipcode,s.email,s.telno,s.faxno,s.telex,s.active,s.remarks,c.id,c.name,c.active,c.remarks FROM suppliers s, countries c WHERE s.country=c.id AND s.id=?";
			stmt = con.prepareStatement (sql);
			
			stmt.setLong (1, id);
			ResultSet rs = stmt.executeQuery ();
			
			if (!rs.next()) return null;
			
			Country country = new Country (
				rs.getLong(13),
				rs.getString(14),
				rs.getBoolean(15),
				rs.getString(16));

			Supplier supplier = new Supplier (
				rs.getLong (1),     // id
				rs.getString (2),   // name
				rs.getString (3),   // contact
				rs.getString (4),   // department
				rs.getString (5),   // address
				rs.getString (6),   // zipcode
				country,
				rs.getString (7),   // email
				rs.getString (8),   // telno
				rs.getString (9),   // faxno
				rs.getString (10),  // telex
				rs.getBoolean (11), // active
				rs.getString (12)); // remarks

			closeStatement (stmt); stmt = null;


			// load categories
			stmt = con.prepareStatement (
				"SELECT c.id,c.name,c.active,c.remarks FROM supplier_category_mappings m, supplier_categories c WHERE m.supplier=? AND m.category=c.id ORDER BY c.name ASC");
			stmt.setLong (1, supplier.getId());

			rs = stmt.executeQuery ();
			Supplier.Categories cs = supplier.new Categories ();

			while (rs.next())
			{
				SupplierCategory c = new SupplierCategory (
					rs.getLong(1),
					rs.getString(2),
					rs.getBoolean(3),
					rs.getString(4));

				cs.add (c);
			}

			if (cs.size() > 0) supplier.setCategories (cs);
			closeStatement (stmt); stmt = null;


			// load ports
			stmt = con.prepareStatement (
				"SELECT p.id,p.name,p.active,p.remarks,c.id,c.name,c.active,c.remarks FROM supplier_port_mappings m, ports p, countries c WHERE m.supplier=? AND m.port=p.id AND p.country=c.id ORDER BY p.name ASC");
			stmt.setLong (1, supplier.getId());

			rs = stmt.executeQuery ();
			Supplier.Ports sp = supplier.new Ports ();

			while (rs.next())
			{
				Country c = new Country (
					rs.getLong(5),
					rs.getString(6),
					rs.getBoolean(7),
					rs.getString(8));

				Port p = new Port (
					rs.getLong(1),
					rs.getString(2),
					c,
					rs.getBoolean(3),
					rs.getString(4));

				sp.add (p);
			}

			if (sp.size() > 0) supplier.setPorts (sp);

			con.commit ();			
			return supplier;
		}
		catch (Exception e)
		{
			rollbackTransaction (con);
			throw new TransactionException (e.getMessage (), e.getCause ());
		}
		finally
		{
			closeStatement (stmt);
			closeConnection (con);
		}
	}
	
	public List getAllSuppliers () throws TransactionException
	{		
		Connection con = getConnection ();
		PreparedStatement stmt = null;
		LinkedList supplierList = new LinkedList ();
		
		try
		{
			stmt = con.prepareStatement (
				"SELECT s.id,s.name,s.contact,s.department,s.address,s.zipcode,s.email,s.telno,s.faxno,s.telex,s.active,s.remarks,c.id,c.name,c.active,c.remarks FROM suppliers s, countries c WHERE s.country=c.id");

			ResultSet rs = stmt.executeQuery ();
			
			while (rs.next ())
			{
				Country country = new Country (
					rs.getLong(13),
					rs.getString(14),
					rs.getBoolean(15),
					rs.getString(16));
	
				Supplier supplier = new Supplier (
					rs.getLong (1),     // id
					rs.getString (2),   // name
					rs.getString (3),   // contact
					rs.getString (4),   // department
					rs.getString (5),   // address
					rs.getString (6),   // zipcode
					country,
					rs.getString (7),   // email
					rs.getString (8),   // telno
					rs.getString (9),   // faxno
					rs.getString (10),  // telex
					rs.getBoolean (11), // active
					rs.getString (12)); // remarks
				
				supplierList.addLast (supplier);
			}
			
			Iterator it = supplierList.iterator ();
			while (it.hasNext())
			{
				Supplier supplier = (Supplier)it.next ();

				closeStatement (stmt); stmt = null;
				stmt = con.prepareStatement (
					"SELECT c.id,c.name,c.active,c.remarks FROM supplier_category_mappings m, supplier_categories c WHERE m.supplier=? AND m.category=c.id ORDER BY c.name DESC");
				stmt.setLong (1, supplier.getId());

				rs = stmt.executeQuery ();
				Supplier.Categories cs = supplier.new Categories ();

				while (rs.next())
				{
					SupplierCategory c = new SupplierCategory (
						rs.getLong(1),
						rs.getString(2),
						rs.getBoolean(3),
						rs.getString(4));

					cs.add (c);
				}

				if (cs.size() > 0) supplier.setCategories (cs);

				closeStatement (stmt); stmt = null;
				stmt = con.prepareStatement (
					"SELECT p.id,p.name,p.active,p.remarks,c.id,c.name,c.active,c.remarks FROM supplier_port_mappings m, ports p, countries c WHERE m.supplier=? AND m.port=p.id AND p.country=c.id ORDER BY p.name ASC");
				stmt.setLong (1, supplier.getId());

				rs = stmt.executeQuery ();
				Supplier.Ports sp = supplier.new Ports ();

				while (rs.next())
				{
					Country c = new Country (
						rs.getLong(5),
						rs.getString(6),
						rs.getBoolean(7),
						rs.getString(8));

					Port p = new Port (
						rs.getLong(1),
						rs.getString(2),
						c,
						rs.getBoolean(3),
						rs.getString(4));

					sp.add (p);
				}

				if (sp.size() > 0) supplier.setPorts (sp);
			}



			con.commit ();			
		}
		catch (Exception e)
		{
			rollbackTransaction (con);
			throw new TransactionException (e.getMessage (), e.getCause ());
		}
		finally
		{
			closeStatement (stmt);
			closeConnection (con);
		}
		
		return supplierList;
	}
	
	public List getSuppliers (int dir, Supplier base, ViewFilter filter) throws TransactionException
	{
		Connection con = getConnection ();
		PreparedStatement stmt = null;
		LinkedList supplierList = new LinkedList ();
		
		try
		{
			StringBuffer sql = new StringBuffer(
				"SELECT s.id,s.name,s.contact,s.department,s.address,s.zipcode,s.email,s.telno,s.faxno,s.telex,s.active,s.remarks,c.id,c.name,c.active,c.remarks FROM suppliers s, countries c");
			StringBuffer whereSql = new StringBuffer("s.country=c.id");
			
			int sortKeyIndex = 0;
			int searchKeyIndex = 0;
			
			int searchKey = filter.getSearchKey ();
			String searchWord = filter.getSearchWord ();
			if (searchWord != null && searchWord.length () > 0)
			{
				if (searchKey == SupplierViewFilter.SEARCH_KEY_NAME)
				{
					if (whereSql.length () > 0) whereSql.append (" AND ");
					whereSql.append ("INSTR(s.name,?)>0");
					searchKeyIndex = 1;
				}
				else if (searchKey == SupplierViewFilter.SEARCH_KEY_CONTACT)
				{
					if (whereSql.length () > 0) whereSql.append (" AND ");
					whereSql.append ("INSTR(s.contact,?)>0");
					searchKeyIndex = 1;
				}
				else if (searchKey == SupplierViewFilter.SEARCH_KEY_DEPARTMENT)
				{
					if (whereSql.length () > 0) whereSql.append (" AND ");
					whereSql.append ("INSTR(s.department,?)>0");
					searchKeyIndex = 1;
				}
				else if (searchKey == SupplierViewFilter.SEARCH_KEY_ADDRESS)
				{
					if (whereSql.length () > 0) whereSql.append (" AND ");
					whereSql.append ("INSTR(s.address,?)>0");
					searchKeyIndex = 1;
				}
				else if (searchKey == SupplierViewFilter.SEARCH_KEY_ZIPCODE)
				{
					if (whereSql.length () > 0) whereSql.append (" AND ");
					whereSql.append ("INSTR(s.zipcode,?)>0");
					searchKeyIndex = 1;
				}
				else if (searchKey == SupplierViewFilter.SEARCH_KEY_COUNTRY)
				{
					if (whereSql.length () > 0) whereSql.append (" AND ");
					whereSql.append ("INSTR(c.name,?)>0");
					searchKeyIndex = 1;
				}
				else if (searchKey == SupplierViewFilter.SEARCH_KEY_EMAIL)
				{
					if (whereSql.length () > 0) whereSql.append (" AND ");
					whereSql.append ("INSTR(s.email,?)>0");
					searchKeyIndex = 1;
				}
				else if (searchKey == SupplierViewFilter.SEARCH_KEY_TELNO)
				{
					if (whereSql.length () > 0) whereSql.append (" AND ");
					whereSql.append ("INSTR(s.telno,?)>0");
					searchKeyIndex = 1;
				}
				else if (searchKey == SupplierViewFilter.SEARCH_KEY_FAXNO)
				{
					if (whereSql.length () > 0) whereSql.append (" AND ");
					whereSql.append ("INSTR(s.faxno,?)>0");
					searchKeyIndex = 1;
				}
				else if (searchKey == SupplierViewFilter.SEARCH_KEY_TELEX)
				{
					if (whereSql.length () > 0) whereSql.append (" AND ");
					whereSql.append ("INSTR(s.telex,?)>0");
					searchKeyIndex = 1;
				}
				else if (searchKey == SupplierViewFilter.SEARCH_KEY_REMARKS)
				{
					if (whereSql.length () > 0) whereSql.append (" AND ");
					whereSql.append ("INSTR(s.remarks,?)>0");
					searchKeyIndex = 1;
				}
			}
			
			int statusKey = filter.getStatusKey ();
			if (statusKey == SupplierViewFilter.STATUS_KEY_ACTIVE)
			{
				if (whereSql.length () > 0) whereSql.append (" AND ");
				whereSql.append ("s.active=1");
			}
			else if (statusKey == SupplierViewFilter.STATUS_KEY_INACTIVE)
			{
				if (whereSql.length () > 0) whereSql.append (" AND ");
				whereSql.append ("s.active=0");
			}
			
			int sortKey = filter.getSortKey ();
			if (sortKey == SupplierViewFilter.SORT_KEY_ID)
			{
				if (whereSql.length () > 0) whereSql.append (" AND ");
				if (dir == GET_NEXT) whereSql.append ("s.id > ? ORDER BY s.id ASC");
				else if (dir == GET_PREVIOUS) whereSql.append ("s.id < ? ORDER BY s.id DESC");
				else whereSql.append ("s.id >= ? ORDER BY s.id ASC");
				
				sortKeyIndex = searchKeyIndex + 1;
			}
			else if (sortKey == SupplierViewFilter.SORT_KEY_NAME) 
			{
				if (whereSql.length () > 0) whereSql.append (" AND ");
				if (dir == GET_NEXT) whereSql.append ("s.name > ? ORDER BY s.name ASC");
				else if (dir == GET_PREVIOUS) whereSql.append ("s.name < ? ORDER BY s.name DESC");
				else whereSql.append ("s.name >= ? ORDER BY s.name ASC");
				
				sortKeyIndex = searchKeyIndex + 1;
			}
			else if (sortKey == SupplierViewFilter.SORT_KEY_EMAIL)
			{
				if (whereSql.length () > 0) whereSql.append (" AND ");
				if (dir == GET_NEXT) whereSql.append ("s.email > ? ORDER BY s.email ASC");
				else if (dir == GET_PREVIOUS) whereSql.append ("s.email < ? ORDER BY s.email DESC");
				else whereSql.append ("s.email >= ? ORDER BY s.email ASC");
				
				sortKeyIndex = searchKeyIndex + 1;
			}
			else if (sortKey == SupplierViewFilter.SORT_KEY_TELNO)
			{
				if (whereSql.length () > 0) whereSql.append (" AND ");
				if (dir == GET_NEXT) whereSql.append ("s.telno > ? ORDER BY s.telno ASC");
				else if (dir == GET_PREVIOUS) whereSql.append ("s.telno < ? ORDER BY s.telno DESC");
				else whereSql.append ("s.telno >= ? ORDER BY s.telno ASC");
				
				sortKeyIndex = searchKeyIndex + 1;
			}
			
			sql.append (" WHERE ");
			sql.append (whereSql);
			
			int limit = filter.getLimit ();
			if (limit > 0) 
			{
				sql.append (" LIMIT ");
				sql.append (limit);
			}
			
			stmt = con.prepareStatement (sql.toString ());
			
			if (sortKey == SupplierViewFilter.SORT_KEY_ID) stmt.setLong (sortKeyIndex, base.getId());			
			else if (sortKey == SupplierViewFilter.SORT_KEY_NAME) stmt.setString (sortKeyIndex, base.getName ());
			else if (sortKey == SupplierViewFilter.SORT_KEY_EMAIL) stmt.setString (sortKeyIndex, base.getEmail ());
			else if (sortKey == SupplierViewFilter.SORT_KEY_TELNO) stmt.setString (sortKeyIndex, base.getTelno ());
			
			if (searchKeyIndex > 0) stmt.setString (searchKeyIndex, searchWord);
			
			ResultSet rs = stmt.executeQuery ();
			
			while (rs.next ())
			{
				Country country = new Country (
					rs.getLong(13),
					rs.getString(14),
					rs.getBoolean(15),
					rs.getString(16));
	
				Supplier supplier = new Supplier (
					rs.getLong (1),     // id
					rs.getString (2),   // name
					rs.getString (3),   // contact
					rs.getString (4),   // department
					rs.getString (5),   // address
					rs.getString (6),   // zipcode
					country,
					rs.getString (7),   // email
					rs.getString (8),   // telno
					rs.getString (9),   // faxno
					rs.getString (10),  // telex
					rs.getBoolean (11), // active
					rs.getString (12)); // remarks
				
				if (dir == GET_PREVIOUS)
					supplierList.addFirst (supplier);
				else supplierList.addLast (supplier);
			}
			
			Iterator it = supplierList.iterator ();
			while (it.hasNext())
			{
				Supplier supplier = (Supplier)it.next ();

				closeStatement (stmt); stmt = null;
				stmt = con.prepareStatement (
					"SELECT c.id,c.name,c.active,c.remarks FROM supplier_category_mappings m, supplier_categories c WHERE m.supplier=? AND m.category=c.id ORDER BY c.name DESC");
				stmt.setLong (1, supplier.getId());

				rs = stmt.executeQuery ();
				Supplier.Categories cs = supplier.new Categories ();

				while (rs.next())
				{
					SupplierCategory c = new SupplierCategory (
						rs.getLong(1),
						rs.getString(2),
						rs.getBoolean(3),
						rs.getString(4));

					cs.add (c);
				}

				if (cs.size() > 0) supplier.setCategories (cs);


				closeStatement (stmt); stmt = null;
				stmt = con.prepareStatement (
					"SELECT p.id,p.name,p.active,p.remarks,c.id,c.name,c.active,c.remarks FROM supplier_port_mappings m, ports p, countries c WHERE m.supplier=? AND m.port=p.id AND p.country=c.id ORDER BY p.name ASC");
				stmt.setLong (1, supplier.getId());

				rs = stmt.executeQuery ();
				Supplier.Ports sp = supplier.new Ports ();

				while (rs.next())
				{
					Country c = new Country (
						rs.getLong(5),
						rs.getString(6),
						rs.getBoolean(7),
						rs.getString(8));

					Port p = new Port (
						rs.getLong(1),
						rs.getString(2),
						c,
						rs.getBoolean(3),
						rs.getString(4));

					sp.add (p);
				}

				if (sp.size() > 0) supplier.setPorts (sp);
			}

			con.commit ();			
		}
		catch (Exception e)
		{
			rollbackTransaction (con);
			throw new TransactionException (e.getMessage (), e.getCause ());
		}
		finally
		{
			closeStatement (stmt);
			closeConnection (con);
		}
		
		return supplierList;
	}
	
	public void deleteSuppliers (long[] ids) throws TransactionException
	{
		deleteRowsByIds ("suppliers", ids);
	}
	
	public void setSupplierStatus (long[] ids, boolean active) throws TransactionException
	{
		setColumnActiveByIds ("suppliers", ids, active);
	}
	
	public void storeSupplier (Supplier supplier) throws TransactionException
	{
		Connection con = getConnection ();
		PreparedStatement stmt = null;

		try
		{
			Country country = supplier.getCountry();

			if (country.getId () == Country.ID_NEW)
			{
				// load the country by the name
				stmt = con.prepareStatement ("SELECT id FROM countries WHERE name=?");
				stmt.setString (1, country.getName());

				ResultSet rs = stmt.executeQuery ();
				if (rs.next()) country.setId (rs.getLong (1));
				closeStatement (stmt); stmt = null;
			}

			Supplier.Categories sc = supplier.getCategories ();
			if (sc != null)
			{
				Iterator it = sc.iterator ();
				while (it.hasNext())
				{
					SupplierCategory category = (SupplierCategory)it.next();

					if (category.getId() == SupplierCategory.ID_NEW)
					{
						stmt = con.prepareStatement ("SELECT id FROM supplier_categories WHERE name=?");
						stmt.setString (1, category.getName());

						ResultSet rs = stmt.executeQuery ();
						if (rs.next()) category.setId (rs.getLong(1));
						closeStatement (stmt); stmt = null;
					}
				}
			}

			Supplier.Ports sp = supplier.getPorts ();	
			if (sp != null)
			{	
				Iterator it = sp.iterator ();
				while (it.hasNext())
				{
					Port port = (Port)it.next();

					if (port.getId() == Port.ID_NEW)
					{
						stmt = con.prepareStatement ("SELECT id FROM ports WHERE name=?");
						stmt.setString (1, port.getName());

						ResultSet rs = stmt.executeQuery ();
						if (rs.next()) port.setId (rs.getLong(1));
						closeStatement (stmt); stmt = null;
					}
				}
			}
			
			long actualSupplierId = supplier.getId();

			if (supplier.getId () == Supplier.ID_NEW)
			{
				stmt = con.prepareStatement ("INSERT INTO suppliers(name,contact,department,address,zipcode,country,email,telno,faxno,telex,active,remarks) VALUES(?,?,?,?,?,?,?,?,?,?,?,?)");
				stmt.setString (1, supplier.getName ());
				stmt.setString (2, supplier.getContact ());
				stmt.setString (3, supplier.getDepartment ());
				stmt.setString (4, supplier.getAddress ());
				stmt.setString (5, supplier.getZipcode ());
				stmt.setLong (6, country.getId());
				stmt.setString (7, supplier.getEmail ());
				stmt.setString (8, supplier.getTelno ());
				stmt.setString (9, supplier.getFaxno ());
				stmt.setString (10, supplier.getTelex ());
				stmt.setBoolean (11, supplier.getActive ());
				stmt.setString (12, supplier.getRemarks ());

				stmt.executeUpdate ();

				ResultSet rs = stmt.getGeneratedKeys ();
				if (rs.next()) actualSupplierId = rs.getLong(1);
			}
			else
			{
				stmt = con.prepareStatement ("UPDATE suppliers SET name=?, contact=?, department=?, address=?, zipcode=?, country=?, email=?, telno=?, faxno=?, telex=?, active=?, remarks=? WHERE id=?");
				stmt.setString (1, supplier.getName ());
				stmt.setString (2, supplier.getContact ());
				stmt.setString (3, supplier.getDepartment ());
				stmt.setString (4, supplier.getAddress ());
				stmt.setString (5, supplier.getZipcode ());
				stmt.setLong (6, country.getId());
				stmt.setString (7, supplier.getEmail ());
				stmt.setString (8, supplier.getTelno ());
				stmt.setString (9, supplier.getFaxno ());
				stmt.setString (10, supplier.getTelex ());
				stmt.setBoolean (11, supplier.getActive ());
				stmt.setString (12, supplier.getRemarks ());
				stmt.setLong (13, supplier.getId ());

				stmt.executeUpdate ();
			}
			

			if (supplier.getId () != Supplier.ID_NEW)
			{
				stmt = con.prepareStatement ("DELETE FROM supplier_category_mappings WHERE supplier=?");
				stmt.setLong (1, actualSupplierId);
				stmt.executeUpdate ();
				closeStatement (stmt); stmt = null;


				stmt = con.prepareStatement ("DELETE FROM supplier_port_mappings WHERE supplier=?");
				stmt.setLong (1, actualSupplierId);
				stmt.executeUpdate ();
				closeStatement (stmt); stmt = null;
			}

			if (sc != null)
			{
				Iterator it = sc.iterator ();
				while (it.hasNext())
				{
					SupplierCategory category = (SupplierCategory)it.next();

					stmt = con.prepareStatement ("INSERT INTO supplier_category_mappings(supplier,category) VALUES(?,?)");
					stmt.setLong (1, actualSupplierId);
					stmt.setLong (2, category.getId());

					stmt.executeUpdate ();
					closeStatement (stmt); stmt = null;
				}
			}

			if (sp != null)
			{				
				Iterator it = sp.iterator ();
				while (it.hasNext())
				{
					Port port = (Port)it.next();

					stmt = con.prepareStatement ("INSERT INTO supplier_port_mappings(supplier,port) VALUES(?,?)");
					stmt.setLong (1, actualSupplierId);
					stmt.setLong (2, port.getId());

					stmt.executeUpdate ();
					closeStatement (stmt); stmt = null;			
				}				
			}

			con.commit ();
		}
		catch (Exception e)
		{
			rollbackTransaction (con);
			throw new TransactionException (e.getMessage (), e.getCause ());
		}
		finally
		{
			closeStatement (stmt);
			closeConnection (con);
		}
	}
	//////////////////////////////////////////////
	// END SUPPLIER
	//////////////////////////////////////////////
	
	//////////////////////////////////////////////
	// BEGIN BUYER
	//////////////////////////////////////////////
	public Buyer getBuyer (long id) throws TransactionException
	{
		Connection con = getConnection ();
		PreparedStatement stmt = null;

		try
		{
			String sql = "SELECT s.id,s.name,s.contact,s.department,s.address,s.zipcode,s.email,s.telno,s.faxno,s.telex,s.active,s.remarks,c.id,c.name,c.active,c.remarks FROM buyers s, countries c WHERE s.country=c.id AND s.id=?";
			stmt = con.prepareStatement (sql);
			
			stmt.setLong (1, id);
			ResultSet rs = stmt.executeQuery ();
			
			if (!rs.next()) return null;
			
			Country country = new Country (
				rs.getLong(13),
				rs.getString(14),
				rs.getBoolean(15),
				rs.getString(16));

			Buyer buyer = new Buyer (
				rs.getLong (1),     // id
				rs.getString (2),   // name
				rs.getString (3),   // contact
				rs.getString (4),   // department
				rs.getString (5),   // address
				rs.getString (6),   // zipcode
				country,
				rs.getString (7),   // email
				rs.getString (8),   // telno
				rs.getString (9),   // faxno
				rs.getString (10),  // telex
				rs.getBoolean (11), // active
				rs.getString (12)); // remarks

			closeStatement (stmt); stmt = null;

			con.commit ();			
			return buyer;
		}
		catch (Exception e)
		{
			rollbackTransaction (con);
			throw new TransactionException (e.getMessage (), e.getCause ());
		}
		finally
		{
			closeStatement (stmt);
			closeConnection (con);
		}
	}
	
	public List getAllBuyers () throws TransactionException
	{		
		Connection con = getConnection ();
		PreparedStatement stmt = null;
		LinkedList buyerList = new LinkedList ();
		
		try
		{
			stmt = con.prepareStatement (
				"SELECT s.id,s.name,s.contact,s.department,s.address,s.zipcode,s.email,s.telno,s.faxno,s.telex,s.active,s.remarks,c.id,c.name,c.active,c.remarks FROM buyers s, countries c WHERE s.country=c.id");

			ResultSet rs = stmt.executeQuery ();
			
			while (rs.next ())
			{
				Country country = new Country (
					rs.getLong(13),
					rs.getString(14),
					rs.getBoolean(15),
					rs.getString(16));
	
				Buyer buyer = new Buyer (
					rs.getLong (1),     // id
					rs.getString (2),   // name
					rs.getString (3),   // contact
					rs.getString (4),   // department
					rs.getString (5),   // address
					rs.getString (6),   // zipcode
					country,
					rs.getString (7),   // email
					rs.getString (8),   // telno
					rs.getString (9),   // faxno
					rs.getString (10),  // telex
					rs.getBoolean (11), // active
					rs.getString (12)); // remarks
				
				buyerList.addLast (buyer);
			}
			
			con.commit ();			
		}
		catch (Exception e)
		{
			rollbackTransaction (con);
			throw new TransactionException (e.getMessage (), e.getCause ());
		}
		finally
		{
			closeStatement (stmt);
			closeConnection (con);
		}
		
		return buyerList;
	}
	
	public List getBuyers (int dir, Buyer base, ViewFilter filter) throws TransactionException
	{
		Connection con = getConnection ();
		PreparedStatement stmt = null;
		LinkedList buyerList = new LinkedList ();
		
		try
		{
			StringBuffer sql = new StringBuffer(
				"SELECT s.id,s.name,s.contact,s.department,s.address,s.zipcode,s.email,s.telno,s.faxno,s.telex,s.active,s.remarks,c.id,c.name,c.active,c.remarks FROM buyers s, countries c");
			StringBuffer whereSql = new StringBuffer("s.country=c.id");
			
			int sortKeyIndex = 0;
			int searchKeyIndex = 0;
			
			int searchKey = filter.getSearchKey ();
			String searchWord = filter.getSearchWord ();
			if (searchWord != null && searchWord.length () > 0)
			{
				if (searchKey == BuyerViewFilter.SEARCH_KEY_NAME)
				{
					if (whereSql.length () > 0) whereSql.append (" AND ");
					whereSql.append ("INSTR(s.name,?)>0");
					searchKeyIndex = 1;
				}
				else if (searchKey == BuyerViewFilter.SEARCH_KEY_CONTACT)
				{
					if (whereSql.length () > 0) whereSql.append (" AND ");
					whereSql.append ("INSTR(s.contact,?)>0");
					searchKeyIndex = 1;
				}
				else if (searchKey == BuyerViewFilter.SEARCH_KEY_DEPARTMENT)
				{
					if (whereSql.length () > 0) whereSql.append (" AND ");
					whereSql.append ("INSTR(s.department,?)>0");
					searchKeyIndex = 1;
				}
				else if (searchKey == BuyerViewFilter.SEARCH_KEY_ADDRESS)
				{
					if (whereSql.length () > 0) whereSql.append (" AND ");
					whereSql.append ("INSTR(s.address,?)>0");
					searchKeyIndex = 1;
				}
				else if (searchKey == BuyerViewFilter.SEARCH_KEY_ZIPCODE)
				{
					if (whereSql.length () > 0) whereSql.append (" AND ");
					whereSql.append ("INSTR(s.zipcode,?)>0");
					searchKeyIndex = 1;
				}
				else if (searchKey == BuyerViewFilter.SEARCH_KEY_COUNTRY)
				{
					if (whereSql.length () > 0) whereSql.append (" AND ");
					whereSql.append ("INSTR(c.name,?)>0");
					searchKeyIndex = 1;
				}
				else if (searchKey == BuyerViewFilter.SEARCH_KEY_EMAIL)
				{
					if (whereSql.length () > 0) whereSql.append (" AND ");
					whereSql.append ("INSTR(s.email,?)>0");
					searchKeyIndex = 1;
				}
				else if (searchKey == BuyerViewFilter.SEARCH_KEY_TELNO)
				{
					if (whereSql.length () > 0) whereSql.append (" AND ");
					whereSql.append ("INSTR(s.telno,?)>0");
					searchKeyIndex = 1;
				}
				else if (searchKey == BuyerViewFilter.SEARCH_KEY_FAXNO)
				{
					if (whereSql.length () > 0) whereSql.append (" AND ");
					whereSql.append ("INSTR(s.faxno,?)>0");
					searchKeyIndex = 1;
				}
				else if (searchKey == BuyerViewFilter.SEARCH_KEY_TELEX)
				{
					if (whereSql.length () > 0) whereSql.append (" AND ");
					whereSql.append ("INSTR(s.telex,?)>0");
					searchKeyIndex = 1;
				}
				else if (searchKey == BuyerViewFilter.SEARCH_KEY_REMARKS)
				{
					if (whereSql.length () > 0) whereSql.append (" AND ");
					whereSql.append ("INSTR(s.remarks,?)>0");
					searchKeyIndex = 1;
				}
			}
			
			int statusKey = filter.getStatusKey ();
			if (statusKey == BuyerViewFilter.STATUS_KEY_ACTIVE)
			{
				if (whereSql.length () > 0) whereSql.append (" AND ");
				whereSql.append ("s.active=1");
			}
			else if (statusKey == BuyerViewFilter.STATUS_KEY_INACTIVE)
			{
				if (whereSql.length () > 0) whereSql.append (" AND ");
				whereSql.append ("s.active=0");
			}
			
			int sortKey = filter.getSortKey ();
			if (sortKey == BuyerViewFilter.SORT_KEY_ID)
			{
				if (whereSql.length () > 0) whereSql.append (" AND ");
				if (dir == GET_NEXT) whereSql.append ("s.id > ? ORDER BY s.id ASC");
				else if (dir == GET_PREVIOUS) whereSql.append ("s.id < ? ORDER BY s.id DESC");
				else whereSql.append ("s.id >= ? ORDER BY s.id ASC");
				
				sortKeyIndex = searchKeyIndex + 1;
			}
			else if (sortKey == BuyerViewFilter.SORT_KEY_NAME) 
			{
				if (whereSql.length () > 0) whereSql.append (" AND ");
				if (dir == GET_NEXT) whereSql.append ("s.name > ? ORDER BY s.name ASC");
				else if (dir == GET_PREVIOUS) whereSql.append ("s.name < ? ORDER BY s.name DESC");
				else whereSql.append ("s.name >= ? ORDER BY s.name ASC");
				
				sortKeyIndex = searchKeyIndex + 1;
			}
			else if (sortKey == BuyerViewFilter.SORT_KEY_EMAIL)
			{
				if (whereSql.length () > 0) whereSql.append (" AND ");
				if (dir == GET_NEXT) whereSql.append ("s.email > ? ORDER BY s.email ASC");
				else if (dir == GET_PREVIOUS) whereSql.append ("s.email < ? ORDER BY s.email DESC");
				else whereSql.append ("s.email >= ? ORDER BY s.email ASC");

				
				sortKeyIndex = searchKeyIndex + 1;
			}
			else if (sortKey == BuyerViewFilter.SORT_KEY_TELNO)
			{
				if (whereSql.length () > 0) whereSql.append (" AND ");
				if (dir == GET_NEXT) whereSql.append ("s.telno > ? ORDER BY s.telno ASC");
				else if (dir == GET_PREVIOUS) whereSql.append ("s.telno < ? ORDER BY s.telno DESC");
				else whereSql.append ("s.telno >= ? ORDER BY s.telno ASC");
				
				sortKeyIndex = searchKeyIndex + 1;
			}
			
			sql.append (" WHERE ");
			sql.append (whereSql);
			
			int limit = filter.getLimit ();
			if (limit > 0) 
			{
				sql.append (" LIMIT ");
				sql.append (limit);
			}
			
			stmt = con.prepareStatement (sql.toString ());
			
			if (sortKey == BuyerViewFilter.SORT_KEY_ID) stmt.setLong (sortKeyIndex, base.getId());			
			else if (sortKey == BuyerViewFilter.SORT_KEY_NAME) stmt.setString (sortKeyIndex, base.getName ());
			else if (sortKey == BuyerViewFilter.SORT_KEY_EMAIL) stmt.setString (sortKeyIndex, base.getEmail ());
			else if (sortKey == BuyerViewFilter.SORT_KEY_TELNO) stmt.setString (sortKeyIndex, base.getTelno ());
			
			if (searchKeyIndex > 0) stmt.setString (searchKeyIndex, searchWord);
			
			ResultSet rs = stmt.executeQuery ();
			
			while (rs.next ())
			{
				Country country = new Country (
					rs.getLong(13),
					rs.getString(14),
					rs.getBoolean(15),
					rs.getString(16));
	
				Buyer buyer = new Buyer (
					rs.getLong (1),     // id
					rs.getString (2),   // name
					rs.getString (3),   // contact
					rs.getString (4),   // department
					rs.getString (5),   // address
					rs.getString (6),   // zipcode
					country,
					rs.getString (7),   // email
					rs.getString (8),   // telno
					rs.getString (9),   // faxno
					rs.getString (10),  // telex
					rs.getBoolean (11), // active
					rs.getString (12)); // remarks
				
				if (dir == GET_PREVIOUS)
					buyerList.addFirst (buyer);
				else buyerList.addLast (buyer);
			}
			
			con.commit ();			
		}
		catch (Exception e)
		{
			rollbackTransaction (con);
			throw new TransactionException (e.getMessage (), e.getCause ());
		}
		finally
		{
			closeStatement (stmt);
			closeConnection (con);
		}
		
		return buyerList;
	}
	
	public void deleteBuyers (long[] ids) throws TransactionException
	{
		deleteRowsByIds ("buyers", ids);
	}
	
	public void setBuyerStatus (long[] ids, boolean active) throws TransactionException
	{
		setColumnActiveByIds ("buyers", ids, active);
	}
	
	public void storeBuyer (Buyer buyer) throws TransactionException
	{
		Connection con = getConnection ();
		PreparedStatement stmt = null;

		try
		{
			Country country = buyer.getCountry();

			if (country.getId () == Country.ID_NEW)
			{
				// load the country by the name
				stmt = con.prepareStatement ("SELECT id FROM countries WHERE name=?");
				stmt.setString (1, country.getName());

				ResultSet rs = stmt.executeQuery ();
				if (rs.next()) country.setId (rs.getLong (1));
				closeStatement (stmt); stmt = null;
			}

			long actualBuyerId = buyer.getId();

			if (buyer.getId () == Buyer.ID_NEW)
			{
				stmt = con.prepareStatement ("INSERT INTO buyers(name,contact,department,address,zipcode,country,email,telno,faxno,telex,active,remarks) VALUES(?,?,?,?,?,?,?,?,?,?,?,?)");
				stmt.setString (1, buyer.getName ());
				stmt.setString (2, buyer.getContact ());
				stmt.setString (3, buyer.getDepartment ());
				stmt.setString (4, buyer.getAddress ());
				stmt.setString (5, buyer.getZipcode ());
				stmt.setLong (6, country.getId());
				stmt.setString (7, buyer.getEmail ());
				stmt.setString (8, buyer.getTelno ());
				stmt.setString (9, buyer.getFaxno ());
				stmt.setString (10, buyer.getTelex ());
				stmt.setBoolean (11, buyer.getActive ());
				stmt.setString (12, buyer.getRemarks ());

				stmt.executeUpdate ();

				ResultSet rs = stmt.getGeneratedKeys ();
				if (rs.next()) actualBuyerId = rs.getLong(1);
			}
			else
			{
				stmt = con.prepareStatement ("UPDATE buyers SET name=?, contact=?, department=?, address=?, zipcode=?, country=?, email=?, telno=?, faxno=?, telex=?, active=?, remarks=? WHERE id=?");
				stmt.setString (1, buyer.getName ());
				stmt.setString (2, buyer.getContact ());
				stmt.setString (3, buyer.getDepartment ());
				stmt.setString (4, buyer.getAddress ());
				stmt.setString (5, buyer.getZipcode ());
				stmt.setLong (6, country.getId());
				stmt.setString (7, buyer.getEmail ());
				stmt.setString (8, buyer.getTelno ());
				stmt.setString (9, buyer.getFaxno ());
				stmt.setString (10, buyer.getTelex ());
				stmt.setBoolean (11, buyer.getActive ());
				stmt.setString (12, buyer.getRemarks ());
				stmt.setLong (13, buyer.getId ());

				stmt.executeUpdate ();
			}
			
			con.commit ();
		}
		catch (Exception e)
		{
			rollbackTransaction (con);
			throw new TransactionException (e.getMessage (), e.getCause ());
		}
		finally
		{
			closeStatement (stmt);
			closeConnection (con);
		}
	}
	//////////////////////////////////////////////
	// END BUYER
	//////////////////////////////////////////////
	
	//////////////////////////////////////////////
	// BEGIN BUYER USER
	//////////////////////////////////////////////
	public BuyerUser getBuyerUser (long id) throws TransactionException
	{
		Connection con = getConnection ();
		PreparedStatement stmt = null;

		try
		{
			String sql = "SELECT u.id,u.userid,u.name,u.email,u.admin,u.active,u.remarks,b.id,b.name,b.contact,b.department,b.address,b.zipcode,b.email,b.telno,b.faxno,b.telex,b.active,b.remarks,c.id,c.name,c.active,c.remarks FROM buyer_users u, buyers b, countries c WHERE u.buyer=b.id AND b.country=c.id AND u.id=?";
			stmt = con.prepareStatement (sql);
			
			stmt.setLong (1, id);
			ResultSet rs = stmt.executeQuery ();
			
			if (!rs.next()) return null;
			
			Country country = new Country (
				rs.getLong (20),     // id
				rs.getString (21),   // name
				rs.getBoolean (22),  // active
				rs.getString (23));  // remarks
			
			Buyer buyer = new Buyer (
				rs.getLong(8),     // id
				rs.getString(9),   // name
				rs.getString(10),  // contact
				rs.getString(11),  // department
				rs.getString(12),  // address
				rs.getString(13),  // zipcode
				country,
				rs.getString(14),  // email
				rs.getString(15),  // telno
				rs.getString(16),  // faxno
				rs.getString(17),  // telex
				rs.getBoolean(18), // active
				rs.getString(19)); // remarks
			
			BuyerUser buyerUser = new BuyerUser (
				rs.getLong (1),    // id
				rs.getString (2),  // userid
				"",                // password
				rs.getString (3),  // name
				rs.getString (4),  // email
				buyer,             // buyer
				rs.getBoolean (5), // admin
				rs.getBoolean (6), // active
				rs.getString (7)); // remarks
			con.commit ();			
			return buyerUser;
		}
		catch (Exception e)
		{
			rollbackTransaction (con);
			throw new TransactionException (e.getMessage (), e.getCause ());
		}
		finally
		{
			closeStatement (stmt);
			closeConnection (con);
		}
	}
	
	public List getBuyerUsers (int dir, BuyerUser base, ViewFilter filter) throws TransactionException
	{
		Connection con = getConnection ();
		PreparedStatement stmt = null;
		LinkedList buyerUserList = new LinkedList ();
		
		try
		{
			StringBuffer sql = new StringBuffer("SELECT u.id,u.userid,u.name,u.email,u.admin,u.active,u.remarks,b.id,b.name,b.contact,b.department,b.address,b.zipcode,b.email,b.telno,b.faxno,b.telex,b.active,b.remarks,c.id,c.name,c.active,c.remarks FROM buyer_users u, buyers b, countries c");
			StringBuffer whereSql = new StringBuffer("u.buyer=b.id AND b.country=c.id");
			
			int sortKeyIndex = 0;
			int searchKeyIndex = 0;
			
			int searchKey = filter.getSearchKey ();
			String searchWord = filter.getSearchWord ();
			if (searchWord != null && searchWord.length () > 0)
			{
				if (searchKey == BuyerUserViewFilter.SEARCH_KEY_USERID)
				{
					if (whereSql.length () > 0) whereSql.append (" AND ");
					whereSql.append ("INSTR(u.userid,?)>0");
					searchKeyIndex = 1;
				}
				else if (searchKey == BuyerUserViewFilter.SEARCH_KEY_NAME)
				{
					if (whereSql.length () > 0) whereSql.append (" AND ");
					whereSql.append ("INSTR(u.name,?)>0");
					searchKeyIndex = 1;
				}
				else if (searchKey == BuyerUserViewFilter.SEARCH_KEY_EMAIL)
				{
					if (whereSql.length () > 0) whereSql.append (" AND ");
					whereSql.append ("INSTR(u.email,?)>0");
					searchKeyIndex = 1;
				}
				else if (searchKey == BuyerUserViewFilter.SEARCH_KEY_BUYER)
				{
					if (whereSql.length () > 0) whereSql.append (" AND ");
					whereSql.append ("INSTR(b.name,?)>0");
					searchKeyIndex = 1;

				}
				else if (searchKey == BuyerUserViewFilter.SEARCH_KEY_REMARKS)
				{
					if (whereSql.length () > 0) whereSql.append (" AND ");
					whereSql.append ("INSTR(u.remarks,?)>0");
					searchKeyIndex = 1;
				}
			}
			
			int statusKey = filter.getStatusKey ();
			if (statusKey == BuyerUserViewFilter.STATUS_KEY_ACTIVE)
			{
				if (whereSql.length () > 0) whereSql.append (" AND ");
				whereSql.append ("u.active=1");
			}
			else if (statusKey == BuyerUserViewFilter.STATUS_KEY_INACTIVE)
			{
				if (whereSql.length () > 0) whereSql.append (" AND ");
				whereSql.append ("u.active=0");
			}
			
			int sortKey = filter.getSortKey ();
			if (sortKey == BuyerUserViewFilter.SORT_KEY_ID)
			{
				if (whereSql.length () > 0) whereSql.append (" AND ");
				if (dir == GET_NEXT) whereSql.append ("u.id > ? ORDER BY id ASC");
				else if (dir == GET_PREVIOUS) whereSql.append ("u.id < ? ORDER BY u.id DESC");
				else whereSql.append ("u.id >= ? ORDER BY u.id ASC");
				
				sortKeyIndex = searchKeyIndex + 1;
			}
			else if (sortKey == BuyerUserViewFilter.SORT_KEY_USERID)
			{
				if (whereSql.length () > 0) whereSql.append (" AND ");
				if (dir == GET_NEXT) whereSql.append ("u.userid > ? ORDER BY u.userid ASC");
				else if (dir == GET_PREVIOUS) whereSql.append ("u.userid < ? ORDER BY u.userid DESC");
				else whereSql.append ("u.userid >= ? ORDER BY u.userid ASC");
				
				sortKeyIndex = searchKeyIndex + 1;
			}
			else if (sortKey == BuyerUserViewFilter.SORT_KEY_NAME) 
			{
				if (whereSql.length () > 0) whereSql.append (" AND ");
				if (dir == GET_NEXT) 
					whereSql.append ("(CONCAT(u.name,u.id) > CONCAT(?,?)) ORDER BY CONCAT(u.name,u.id) ASC");
				else if (dir == GET_PREVIOUS) 
					whereSql.append ("(CONCAT(u.name,u.id) < CONCAT(?,?)) ORDER BY CONCAT(u.name,u.id) DESC");
				else whereSql.append ("(CONCAT(u.name,u.id) >= CONCAT(?,?)) ORDER BY CONCAT(u.name,u.id) ASC");
				
				sortKeyIndex = searchKeyIndex + 1;
			}
			else if (sortKey == BuyerUserViewFilter.SORT_KEY_EMAIL) 
			{
				if (whereSql.length () > 0) whereSql.append (" AND ");
				if (dir == GET_NEXT) whereSql.append ("u.email > ? ORDER BY u.email ASC");
				else if (dir == GET_PREVIOUS) whereSql.append ("u.email < ? ORDER BY u.email DESC");
				else whereSql.append ("u.email >= ? ORDER BY u.email ASC");
				
				sortKeyIndex = searchKeyIndex + 1;
			}
			else if (sortKey == BuyerUserViewFilter.SORT_KEY_BUYER) 
			{
				if (whereSql.length () > 0) whereSql.append (" AND ");
				if (dir == GET_NEXT) 
					whereSql.append ("CONCAT(b.name,u.id) > CONCAT(?,?) ORDER BY CONCAT(b.name,u.id) ASC");
				else if (dir == GET_PREVIOUS) 
					whereSql.append ("CONCAT(b.name,u.id) < CONCAT(?,?) ORDER BY CONCAT(b.name,u.id) DESC");
				else whereSql.append ("CONCAT(b.name,u.id) >= CONCAT(?,?) ORDER BY CONCAT(b.name,u.id) ASC");

				sortKeyIndex = searchKeyIndex + 1;

			}
			
			sql.append (" WHERE ");
			sql.append (whereSql);

			int limit = filter.getLimit ();
			if (limit > 0) 
			{
				sql.append (" LIMIT ");
				sql.append (limit);
			}
			
			stmt = con.prepareStatement (sql.toString ());
			
			if (sortKey == BuyerUserViewFilter.SORT_KEY_ID) 
			{
				stmt.setLong (sortKeyIndex, base.getId());
			}
			else if (sortKey == BuyerUserViewFilter.SORT_KEY_USERID)  
			{
				stmt.setString (sortKeyIndex, base.getUserid ());
			}
			else if (sortKey == BuyerUserViewFilter.SORT_KEY_NAME)  
			{
				stmt.setString (sortKeyIndex, base.getName ());
				stmt.setLong (sortKeyIndex+1, base.getId ());
			}
			else if (sortKey == BuyerUserViewFilter.SORT_KEY_EMAIL)
			{
				stmt.setString (sortKeyIndex, base.getName ());
			}
			else if (sortKey == BuyerUserViewFilter.SORT_KEY_BUYER)
			{
				stmt.setString (sortKeyIndex, base.getBuyer().getName ());
				stmt.setLong (sortKeyIndex+1, base.getId ());
			}
			
			if (searchKeyIndex > 0) stmt.setString (searchKeyIndex, searchWord);
			
			ResultSet rs = stmt.executeQuery ();
			
			while (rs.next ())
			{
				Country country = new Country (
					rs.getLong (20),     // id
					rs.getString (21),   // name
					rs.getBoolean (22),  // active
					rs.getString (23));  // remarks
			
				Buyer buyer = new Buyer (
					rs.getLong(8),     // id
					rs.getString(9),   // name
					rs.getString(10),  // contact
					rs.getString(11),  // department
					rs.getString(12),  // address
					rs.getString(13),  // zipcode
					country,
					rs.getString(14),  // email
					rs.getString(15),  // telno
					rs.getString(16),  // faxno
					rs.getString(17),  // telex
					rs.getBoolean(18), // active
					rs.getString(19)); // remarks
			
				BuyerUser buyerUser = new BuyerUser (
					rs.getLong (1),    // id
					rs.getString (2),  // userid
					"",                // password
					rs.getString (3),  // name
					rs.getString (4),  // email
					buyer,             // buyer
					rs.getBoolean (5), // admin
					rs.getBoolean (6), // active
					rs.getString (7)); // remarks
	
				if (dir == GET_PREVIOUS)
					buyerUserList.addFirst (buyerUser);
				else buyerUserList.addLast (buyerUser);
			}
			
			con.commit ();			
		}
		catch (Exception e)
		{
			rollbackTransaction (con);
			throw new TransactionException (e.getMessage (), e.getCause ());
		}
		finally
		{
			closeStatement (stmt);
			closeConnection (con);
		}
		
		return buyerUserList;
	}
	
	public void deleteBuyerUsers (long[] ids) throws TransactionException
	{
		deleteRowsByIds ("buyer_users", ids);
	}
	
	public void setBuyerUserStatus (
		long[] ids, boolean active) throws TransactionException
	{
		setColumnActiveByIds ("buyer_users", ids, active);
	}
	
	public void storeBuyerUser (BuyerUser buyerUser) throws TransactionException
	{
		Connection con = getConnection ();
		PreparedStatement stmt = null;
		
		try
		{
			Buyer buyer = buyerUser.getBuyer ();
			if (buyer.getId () == Buyer.ID_NEW)
			{
				// load a buyer by the name
				stmt = con.prepareStatement ("SELECT id FROM buyers WHERE name=?");
				stmt.setString (1, buyer.getName());

				ResultSet rs = stmt.executeQuery ();
				if (rs.next())
				{
					buyer.setId (rs.getLong (1));
				}
				closeStatement (stmt); stmt = null;
			}
			
			if (buyerUser.getId () == BuyerUser.ID_NEW)
			{
				MD5 md5 = new MD5();
				String pwd = buyerUser.getUserid () + buyerUser.getPassword ();
				String hashedPwd = md5.hashData (pwd.getBytes ("utf-8"));
				
				stmt = con.prepareStatement ("INSERT INTO buyer_users(userid,password,name,email,buyer,admin,active,remarks) VALUES(?,?,?,?,?,?,?,?)");
				stmt.setString (1, buyerUser.getUserid ());
				stmt.setString (2, hashedPwd);
				stmt.setString (3, buyerUser.getName ());
				stmt.setString (4, buyerUser.getEmail ());
				stmt.setLong (5, buyer.getId ());
				stmt.setBoolean (6, buyerUser.getAdmin ());
				stmt.setBoolean (7, buyerUser.getActive ());
				stmt.setString (8, buyerUser.getRemarks ());
			}
			else
			{
				String passwd = buyerUser.getPassword();
				String sql;

				if (passwd == null)
				{
					sql = "UPDATE buyer_users SET userid=?, name=?, email=?, buyer=?, admin=?, active=?, remarks=? WHERE id=?";
				}
				else
				{
					sql = "UPDATE buyer_users SET userid=?, password=?, name=?, email=?, buyer=?, admin=?, active=?, remarks=? WHERE id=?";
				}

				stmt = con.prepareStatement (sql);

				int fid = 1;

				stmt.setString (fid++, buyerUser.getUserid ());
				if (passwd != null) 
				{
					MD5 md5 = new MD5();
					String hashedPwd = md5.hashData ((buyerUser.getUserid () + passwd).getBytes ("utf-8"));
					stmt.setString (fid++, hashedPwd);
				}
				stmt.setString (fid++, buyerUser.getName ());
				stmt.setString (fid++, buyerUser.getEmail ());
				stmt.setLong (fid++, buyer.getId());
				stmt.setBoolean (fid++, buyerUser.getAdmin ());
				stmt.setBoolean (fid++, buyerUser.getActive ());
				stmt.setString (fid++, buyerUser.getRemarks ());
				stmt.setLong (fid++, buyerUser.getId ());
			}
			
			stmt.executeUpdate ();
			
			//ResultSet rs = stmt.getGeneratedKeys ();
			con.commit ();
		}
		catch (Exception e)
		{
			rollbackTransaction (con);			
			throw new TransactionException (e.getMessage (), e.getCause ());
		}
		finally
		{			
			closeStatement (stmt);
			closeConnection (con);
		}
	}
	// END BUYER USER
	
	//////////////////////////////////////////////
	// BEGIN SUPPLIER USER
	//////////////////////////////////////////////
	public SupplierUser getSupplierUser (long id) throws TransactionException
	{
		Connection con = getConnection ();
		PreparedStatement stmt = null;

		try
		{
			String sql = "SELECT u.id,u.userid,u.name,u.email,u.admin,u.active,u.remarks,b.id,b.name,b.contact,b.department,b.address,b.zipcode,b.email,b.telno,b.faxno,b.telex,b.active,b.remarks,c.id,c.name,c.active,c.remarks FROM supplier_users u, suppliers b, countries c WHERE u.supplier=b.id AND b.country=c.id AND u.id=?";
			stmt = con.prepareStatement (sql);
			
			stmt.setLong (1, id);
			ResultSet rs = stmt.executeQuery ();
			
			if (!rs.next()) return null;
			
			Country country = new Country (
				rs.getLong (20),     // id
				rs.getString (21),   // name
				rs.getBoolean (22),  // active
				rs.getString (23));  // remarks
			
			Supplier supplier = new Supplier (
				rs.getLong(8),     // id
				rs.getString(9),   // name
				rs.getString(10),  // contact
				rs.getString(11),  // department
				rs.getString(12),  // address
				rs.getString(13),  // zipcode
				country,
				rs.getString(14),  // email
				rs.getString(15),  // telno
				rs.getString(16),  // faxno
				rs.getString(17),  // telex
				rs.getBoolean(18), // active
				rs.getString(19)); // remarks
			
			SupplierUser supplierUser = new SupplierUser (
				rs.getLong (1),    // id
				rs.getString (2),  // userid
				"",                // password
				rs.getString (3),  // name
				rs.getString (4),  // email
				supplier,             // supplier
				rs.getBoolean (5), // admin
				rs.getBoolean (6), // active
				rs.getString (7)); // remarks
			con.commit ();			
			return supplierUser;
		}
		catch (Exception e)
		{
			rollbackTransaction (con);
			throw new TransactionException (e.getMessage (), e.getCause ());
		}
		finally
		{
			closeStatement (stmt);
			closeConnection (con);
		}
	}
	
	public List getSupplierUsers (int dir, SupplierUser base, ViewFilter filter) throws TransactionException
	{
		Connection con = getConnection ();
		PreparedStatement stmt = null;
		LinkedList supplierUserList = new LinkedList ();
		
		try
		{
			StringBuffer sql = new StringBuffer("SELECT u.id,u.userid,u.name,u.email,u.admin,u.active,u.remarks,b.id,b.name,b.contact,b.department,b.address,b.zipcode,b.email,b.telno,b.faxno,b.telex,b.active,b.remarks,c.id,c.name,c.active,c.remarks FROM supplier_users u, suppliers b, countries c");
			StringBuffer whereSql = new StringBuffer("u.supplier=b.id AND b.country=c.id");
			
			int sortKeyIndex = 0;
			int searchKeyIndex = 0;
			
			int searchKey = filter.getSearchKey ();
			String searchWord = filter.getSearchWord ();
			if (searchWord != null && searchWord.length () > 0)
			{
				if (searchKey == SupplierUserViewFilter.SEARCH_KEY_USERID)
				{
					if (whereSql.length () > 0) whereSql.append (" AND ");
					whereSql.append ("INSTR(u.userid,?)>0");
					searchKeyIndex = 1;
				}
				else if (searchKey == SupplierUserViewFilter.SEARCH_KEY_NAME)
				{
					if (whereSql.length () > 0) whereSql.append (" AND ");
					whereSql.append ("INSTR(u.name,?)>0");
					searchKeyIndex = 1;
				}
				else if (searchKey == SupplierUserViewFilter.SEARCH_KEY_EMAIL)
				{
					if (whereSql.length () > 0) whereSql.append (" AND ");
					whereSql.append ("INSTR(u.email,?)>0");
					searchKeyIndex = 1;
				}
				else if (searchKey == SupplierUserViewFilter.SEARCH_KEY_SUPPLIER)
				{
					if (whereSql.length () > 0) whereSql.append (" AND ");
					whereSql.append ("INSTR(b.name,?)>0");
					searchKeyIndex = 1;

				}
				else if (searchKey == SupplierUserViewFilter.SEARCH_KEY_REMARKS)
				{
					if (whereSql.length () > 0) whereSql.append (" AND ");
					whereSql.append ("INSTR(u.remarks,?)>0");
					searchKeyIndex = 1;
				}
			}
			
			int statusKey = filter.getStatusKey ();
			if (statusKey == SupplierUserViewFilter.STATUS_KEY_ACTIVE)
			{
				if (whereSql.length () > 0) whereSql.append (" AND ");
				whereSql.append ("u.active=1");
			}
			else if (statusKey == SupplierUserViewFilter.STATUS_KEY_INACTIVE)
			{
				if (whereSql.length () > 0) whereSql.append (" AND ");
				whereSql.append ("u.active=0");
			}
			
			int sortKey = filter.getSortKey ();
			if (sortKey == SupplierUserViewFilter.SORT_KEY_ID)
			{
				if (whereSql.length () > 0) whereSql.append (" AND ");
				if (dir == GET_NEXT) whereSql.append ("u.id > ? ORDER BY id ASC");
				else if (dir == GET_PREVIOUS) whereSql.append ("u.id < ? ORDER BY u.id DESC");
				else whereSql.append ("u.id >= ? ORDER BY u.id ASC");
				
				sortKeyIndex = searchKeyIndex + 1;
			}
			else if (sortKey == SupplierUserViewFilter.SORT_KEY_USERID)
			{
				if (whereSql.length () > 0) whereSql.append (" AND ");
				if (dir == GET_NEXT) whereSql.append ("u.userid > ? ORDER BY u.userid ASC");
				else if (dir == GET_PREVIOUS) whereSql.append ("u.userid < ? ORDER BY u.userid DESC");
				else whereSql.append ("u.userid >= ? ORDER BY u.userid ASC");
				
				sortKeyIndex = searchKeyIndex + 1;
			}
			else if (sortKey == SupplierUserViewFilter.SORT_KEY_NAME) 
			{
				if (whereSql.length () > 0) whereSql.append (" AND ");
				if (dir == GET_NEXT) 
					whereSql.append ("(CONCAT(u.name,u.id) > CONCAT(?,?)) ORDER BY CONCAT(u.name,u.id) ASC");
				else if (dir == GET_PREVIOUS) 
					whereSql.append ("(CONCAT(u.name,u.id) < CONCAT(?,?)) ORDER BY CONCAT(u.name,u.id) DESC");
				else whereSql.append ("(CONCAT(u.name,u.id) >= CONCAT(?,?)) ORDER BY CONCAT(u.name,u.id) ASC");
				
				sortKeyIndex = searchKeyIndex + 1;
			}
			else if (sortKey == SupplierUserViewFilter.SORT_KEY_EMAIL) 
			{
				if (whereSql.length () > 0) whereSql.append (" AND ");
				if (dir == GET_NEXT) whereSql.append ("u.email > ? ORDER BY u.email ASC");
				else if (dir == GET_PREVIOUS) whereSql.append ("u.email < ? ORDER BY u.email DESC");
				else whereSql.append ("u.email >= ? ORDER BY u.email ASC");
				
				sortKeyIndex = searchKeyIndex + 1;
			}
			else if (sortKey == SupplierUserViewFilter.SORT_KEY_SUPPLIER) 
			{
				if (whereSql.length () > 0) whereSql.append (" AND ");
				if (dir == GET_NEXT) 
					whereSql.append ("CONCAT(b.name,u.id) > CONCAT(?,?) ORDER BY CONCAT(b.name,u.id) ASC");
				else if (dir == GET_PREVIOUS) 
					whereSql.append ("CONCAT(b.name,u.id) < CONCAT(?,?) ORDER BY CONCAT(b.name,u.id) DESC");
				else whereSql.append ("CONCAT(b.name,u.id) >= CONCAT(?,?) ORDER BY CONCAT(b.name,u.id) ASC");

				sortKeyIndex = searchKeyIndex + 1;

			}
			
			sql.append (" WHERE ");
			sql.append (whereSql);

			int limit = filter.getLimit ();
			if (limit > 0) 
			{
				sql.append (" LIMIT ");
				sql.append (limit);
			}
			
			stmt = con.prepareStatement (sql.toString ());
			
			if (sortKey == SupplierUserViewFilter.SORT_KEY_ID) 
			{
				stmt.setLong (sortKeyIndex, base.getId());
			}
			else if (sortKey == SupplierUserViewFilter.SORT_KEY_USERID)  
			{
				stmt.setString (sortKeyIndex, base.getUserid ());
			}
			else if (sortKey == SupplierUserViewFilter.SORT_KEY_NAME)  
			{
				stmt.setString (sortKeyIndex, base.getName ());
				stmt.setLong (sortKeyIndex+1, base.getId ());
			}
			else if (sortKey == SupplierUserViewFilter.SORT_KEY_EMAIL)
			{
				stmt.setString (sortKeyIndex, base.getName ());
			}
			else if (sortKey == SupplierUserViewFilter.SORT_KEY_SUPPLIER)
			{
				stmt.setString (sortKeyIndex, base.getSupplier().getName ());
				stmt.setLong (sortKeyIndex+1, base.getId ());
			}
			
			if (searchKeyIndex > 0) stmt.setString (searchKeyIndex, searchWord);
			
			ResultSet rs = stmt.executeQuery ();
			
			while (rs.next ())
			{
				Country country = new Country (
					rs.getLong (20),     // id
					rs.getString (21),   // name
					rs.getBoolean (22),  // active
					rs.getString (23));  // remarks
			
				Supplier supplier = new Supplier (
					rs.getLong(8),     // id
					rs.getString(9),   // name
					rs.getString(10),  // contact
					rs.getString(11),  // department
					rs.getString(12),  // address
					rs.getString(13),  // zipcode
					country,
					rs.getString(14),  // email
					rs.getString(15),  // telno
					rs.getString(16),  // faxno
					rs.getString(17),  // telex
					rs.getBoolean(18), // active
					rs.getString(19)); // remarks
			
				SupplierUser supplierUser = new SupplierUser (
					rs.getLong (1),    // id
					rs.getString (2),  // userid
					"",                // password
					rs.getString (3),  // name
					rs.getString (4),  // email
					supplier,             // supplier
					rs.getBoolean (5), // admin
					rs.getBoolean (6), // active
					rs.getString (7)); // remarks
	
				if (dir == GET_PREVIOUS)
					supplierUserList.addFirst (supplierUser);
				else supplierUserList.addLast (supplierUser);
			}
			
			con.commit ();			
		}
		catch (Exception e)
		{
			rollbackTransaction (con);
			throw new TransactionException (e.getMessage (), e.getCause ());
		}
		finally
		{
			closeStatement (stmt);
			closeConnection (con);
		}
		
		return supplierUserList;
	}
	
	public void deleteSupplierUsers (long[] ids) throws TransactionException
	{
		deleteRowsByIds ("supplier_users", ids);
	}
	
	public void setSupplierUserStatus (
		long[] ids, boolean active) throws TransactionException
	{
		setColumnActiveByIds ("supplier_users", ids, active);
	}
	
	public void storeSupplierUser (SupplierUser supplierUser) throws TransactionException
	{
		Connection con = getConnection ();
		PreparedStatement stmt = null;
		
		try
		{
			Supplier supplier = supplierUser.getSupplier ();
			if (supplier.getId () == Supplier.ID_NEW)
			{
				// load a supplier by the name
				stmt = con.prepareStatement ("SELECT id FROM suppliers WHERE name=?");
				stmt.setString (1, supplier.getName());

				ResultSet rs = stmt.executeQuery ();
				if (rs.next())
				{
					supplier.setId (rs.getLong (1));
				}
				closeStatement (stmt); stmt = null;
			}
			
			if (supplierUser.getId () == SupplierUser.ID_NEW)
			{
				MD5 md5 = new MD5();
				String pwd = supplierUser.getUserid () + supplierUser.getPassword ();
				String hashedPwd = md5.hashData (pwd.getBytes ("utf-8"));
				
				stmt = con.prepareStatement ("INSERT INTO supplier_users(userid,password,name,email,supplier,admin,active,remarks) VALUES(?,?,?,?,?,?,?,?)");
				stmt.setString (1, supplierUser.getUserid ());
				stmt.setString (2, hashedPwd);
				stmt.setString (3, supplierUser.getName ());
				stmt.setString (4, supplierUser.getEmail ());
				stmt.setLong (5, supplier.getId ());
				stmt.setBoolean (6, supplierUser.getAdmin ());
				stmt.setBoolean (7, supplierUser.getActive ());
				stmt.setString (8, supplierUser.getRemarks ());
			}
			else
			{
				String passwd = supplierUser.getPassword();
				String sql;

				if (passwd == null)
				{
					sql = "UPDATE supplier_users SET userid=?, name=?, email=?, supplier=?, admin=?, active=?, remarks=? WHERE id=?";
				}
				else
				{
					sql = "UPDATE supplier_users SET userid=?, password=?, name=?, email=?, supplier=?, admin=?, active=?, remarks=? WHERE id=?";
				}

				stmt = con.prepareStatement (sql);

				int fid = 1;

				stmt.setString (fid++, supplierUser.getUserid ());
				if (passwd != null) 
				{
					MD5 md5 = new MD5();
					String hashedPwd = md5.hashData ((supplierUser.getUserid () + passwd).getBytes ("utf-8"));
					stmt.setString (fid++, hashedPwd);
				}
				stmt.setString (fid++, supplierUser.getName ());
				stmt.setString (fid++, supplierUser.getEmail ());
				stmt.setLong (fid++, supplier.getId());
				stmt.setBoolean (fid++, supplierUser.getAdmin ());
				stmt.setBoolean (fid++, supplierUser.getActive ());
				stmt.setString (fid++, supplierUser.getRemarks ());
				stmt.setLong (fid++, supplierUser.getId ());
			}
			
			stmt.executeUpdate ();
			
			//ResultSet rs = stmt.getGeneratedKeys ();
			con.commit ();
		}
		catch (Exception e)
		{
			rollbackTransaction (con);			
			throw new TransactionException (e.getMessage (), e.getCause ());
		}
		finally
		{			
			closeStatement (stmt);
			closeConnection (con);
		}
	}
	// END SUPPLIER USE
	
	//////////////////////////////////////////////
	// BEGIN PRODUCT 
	//////////////////////////////////////////////
	public Product getProduct (long id) throws TransactionException
	{
		Connection con = getConnection ();
		PreparedStatement stmt = null;

		try
		{
			String sql = "SELECT id,code,name,unit,active,remarks FROM products WHERE id=?";
			stmt = con.prepareStatement (sql);
			
			stmt.setLong (1, id);
			ResultSet rs = stmt.executeQuery ();
			
			if (!rs.next()) return null;
			
			Product product = new Product (
					rs.getLong (1),
					rs.getString (2),
					rs.getString (3),
					rs.getString (4),
					rs.getBoolean (5),
					rs.getString (6));
			con.commit ();			
			return product;
		}
		catch (Exception e)
		{
			rollbackTransaction (con);
			throw new TransactionException (e.getMessage (), e.getCause ());
		}
		finally
		{
			closeStatement (stmt);
			closeConnection (con);
		}
	}
	
	public List getProducts (int dir, Product base, ViewFilter filter) throws TransactionException
	{
		Connection con = getConnection ();
		PreparedStatement stmt = null;
		LinkedList productList = new LinkedList ();
		
		try
		{
			StringBuffer sql = new StringBuffer("SELECT id,code,name,unit,active,remarks FROM products");
			StringBuffer whereSql = new StringBuffer();
			
			int sortKeyIndex = 0;
			int searchKeyIndex = 0;
			
			int searchKey = filter.getSearchKey ();
			String searchWord = filter.getSearchWord ();
			if (searchWord != null && searchWord.length () > 0)
			{
				if (searchKey == ProductViewFilter.SEARCH_KEY_CODE)
				{
					whereSql.append ("INSTR(code,?)>0");
					searchKeyIndex = 1;
				}
				else if (searchKey == ProductViewFilter.SEARCH_KEY_NAME)
				{
					whereSql.append ("INSTR(name,?)>0");
					searchKeyIndex = 1;
				}
				else if (searchKey == ProductViewFilter.SEARCH_KEY_UNIT)
				{
					whereSql.append ("INSTR(unit,?)>0");
					searchKeyIndex = 1;
				}
				else if (searchKey == ProductViewFilter.SEARCH_KEY_REMARKS)
				{
					whereSql.append ("INSTR(remarks,?)>0");
					searchKeyIndex = 1;
				}
			}
			
			int statusKey = filter.getStatusKey ();
			if (statusKey == ProductViewFilter.STATUS_KEY_ACTIVE)
			{
				if (whereSql.length () > 0) whereSql.append (" AND ");
				whereSql.append ("active=1");
			}
			else if (statusKey == ProductViewFilter.STATUS_KEY_INACTIVE)
			{
				if (whereSql.length () > 0) whereSql.append (" AND ");
				whereSql.append ("active=0");
			}
			
			int sortKey = filter.getSortKey ();
			if (sortKey == ProductViewFilter.SORT_KEY_ID)
			{
				if (whereSql.length () > 0) whereSql.append (" AND ");
				if (dir == GET_NEXT) whereSql.append ("id > ? ORDER BY id ASC");
				else if (dir == GET_PREVIOUS) whereSql.append ("id < ? ORDER BY id DESC");
				else whereSql.append ("id >= ? ORDER BY id ASC");
				
				sortKeyIndex = searchKeyIndex + 1;
			}
			else if (sortKey == ProductViewFilter.SORT_KEY_CODE)
			{
				if (whereSql.length () > 0) whereSql.append (" AND ");
				if (dir == GET_NEXT) whereSql.append ("code > ? ORDER BY code ASC");
				else if (dir == GET_PREVIOUS) whereSql.append ("code < ? ORDER BY code DESC");
				else whereSql.append ("code >= ? ORDER BY code ASC");
				
				sortKeyIndex = searchKeyIndex + 1;
			}
			else if (sortKey == ProductViewFilter.SORT_KEY_NAME) 
			{
				if (whereSql.length () > 0) whereSql.append (" AND ");
				if (dir == GET_NEXT) 
					whereSql.append ("(CONCAT(name,id) > CONCAT(?,?)) ORDER BY CONCAT(name,id) ASC");
				else if (dir == GET_PREVIOUS) 
					whereSql.append ("(CONCAT(name,id) < CONCAT(?,?)) ORDER BY CONCAT(name,id) DESC");
				else whereSql.append ("(CONCAT(name,id) >= CONCAT(?,?)) ORDER BY CONCAT(name,id) ASC");
				
				sortKeyIndex = searchKeyIndex + 1;
			}
			else /* if (sortKey == ProductViewFilter.SORT_KEY_UNIT) */
			{
				if (whereSql.length () > 0) whereSql.append (" AND ");
				if (dir == GET_NEXT) whereSql.append ("unit > ? ORDER BY unit ASC");
				else if (dir == GET_PREVIOUS) whereSql.append ("unit < ? ORDER BY unit DESC");
				else whereSql.append ("unit >= ? ORDER BY unit ASC");
				
				sortKeyIndex = searchKeyIndex + 1;
			}
			
			sql.append (" WHERE ");
			sql.append (whereSql);

			int limit = filter.getLimit ();
			if (limit > 0) 
			{
				sql.append (" LIMIT ");
				sql.append (limit);
			}
			
			stmt = con.prepareStatement (sql.toString ());
			
			if (sortKey == ProductViewFilter.SORT_KEY_ID) 
			{
				stmt.setLong (sortKeyIndex, base.getId());
			}
			else if (sortKey == ProductViewFilter.SORT_KEY_CODE)  
			{
				stmt.setString (sortKeyIndex, base.getCode ());
			}
			else if (sortKey == ProductViewFilter.SORT_KEY_NAME)  
			{
				stmt.setString (sortKeyIndex, base.getName ());
				stmt.setLong (sortKeyIndex+1, base.getId ());
			}
			else /*(sortKey == ProductViewFilter.SORT_KEY_UNIT)*/  
			{
				stmt.setString (sortKeyIndex, base.getName ());
			}
			
			if (searchKeyIndex > 0) stmt.setString (searchKeyIndex, searchWord);
			
			ResultSet rs = stmt.executeQuery ();
			
			while (rs.next ())
			{
				Product product = new Product (
					rs.getLong (1),
					rs.getString (2),
					rs.getString (3),
					rs.getString (4),
					rs.getBoolean (5),
					rs.getString (6));
				
				if (dir == GET_PREVIOUS)
					productList.addFirst (product);
				else productList.addLast (product);
			}
			
			con.commit ();			
		}
		catch (Exception e)
		{
			rollbackTransaction (con);
			throw new TransactionException (e.getMessage (), e.getCause ());
		}
		finally
		{
			closeStatement (stmt);
			closeConnection (con);
		}
		
		return productList;
	}
	
	public void deleteProducts (long[] ids) throws TransactionException
	{
		deleteRowsByIds ("products", ids);
	}
	
	public void setProductStatus (
		long[] ids, boolean active) throws TransactionException
	{
		setColumnActiveByIds ("products", ids, active);
	}
	
	public void storeProduct (Product product) throws TransactionException
	{
		Connection con = getConnection ();
		PreparedStatement stmt = null;
		
		try
		{
			if (product.getId () == Product.ID_NEW)
			{
				stmt = con.prepareStatement ("INSERT INTO products(code,name,unit,active,remarks) VALUES(?,?,?,?,?)");
				stmt.setString (1, product.getCode ());
				stmt.setString (2, product.getName ());
				stmt.setString (3, product.getUnit ());
				stmt.setBoolean (4, product.getActive ());
				stmt.setString (5, product.getRemarks ());
			}
			else
			{
				String sql = "UPDATE products SET code=?, name=?, unit=?, active=?, remarks=? WHERE id=?";

				stmt = con.prepareStatement (sql);

				int fid = 1;

				stmt.setString (fid++, product.getCode ());
				stmt.setString (fid++, product.getName ());
				stmt.setString (fid++, product.getUnit ());
				stmt.setBoolean (fid++, product.getActive ());
				stmt.setString (fid++, product.getRemarks ());
				stmt.setLong (fid++, product.getId ());
			}
			
			stmt.executeUpdate ();
			
			//ResultSet rs = stmt.getGeneratedKeys ();
			con.commit ();
		}
		catch (Exception e)
		{
			rollbackTransaction (con);			
			throw new TransactionException (e.getMessage (), e.getCause ());
		}
		finally
		{			
			closeStatement (stmt);
			closeConnection (con);
		}
	}
	// END PRODUCT 
}

