/*
 * $Id: Transaction.java,v 1.4 2007/09/19 15:34:20 bacon Exp $
 */

package com.unimar.shipneeds.trans;

import com.unimar.shipneeds.util.MD5;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.sql.DataSource;

public class Transaction
{
	public final static int GET_PREVIOUS = -1;
	public final static int GET_CURRENT = 0;
	public final static int GET_NEXT = 1;
	
	private static Context initialContext = null;

	public Transaction ()
	{
	}
	
	protected void login (String table, String userid, String password) throws TransactionException
	{
		Connection con = getConnection ();
		PreparedStatement stmt = null;
		
		try
		{
			stmt = con.prepareStatement ("SELECT userid, password, active FROM " + table + " WHERE userid=?");
			stmt.setString (1, userid);
			ResultSet rs = stmt.executeQuery ();
			
			if (!rs.next ())
			{
				throw new TransactionNoSuchUserException ("no such user");
			}
			
			if (!rs.getBoolean (3))
			{
				throw new TransactionInactiveUserException ("inactive user");
			}
				
			String pwd = rs.getString(2);
			if (pwd.length () != 0 || password.length() != 0)
			{
				MD5 md5 = new MD5();
				String hashed = md5.hashData ((userid+password).getBytes ("utf-8"));
		
				if (!pwd.equals(hashed))	
				{
					throw new TransactionException ("password wrong");
				}
			}
			
			con.commit ();
		}
		catch (Exception e)
		{
			rollbackTransaction (con);
			throw new TransactionException (e.getMessage (), e.getCause ());
		}
		finally
		{
			closeStatement (stmt);
			closeConnection (con);
		}
	}
	
	protected Connection getConnection () throws TransactionStorageException
	{
		try
		{
			DataSource ds = getDataSource ();
			if (ds == null) throw new TransactionStorageException ("no storage found");
			
			Connection con = ds.getConnection ();
			con.setAutoCommit (false);
			return con;
		}
		catch (SQLException e)
		{
			throw new TransactionStorageException ("cannot get connection to database");
		}
		catch (Exception e)
		{                    
			throw new TransactionStorageException (e.getMessage (), e.getCause ());
		}
	}
	
	protected synchronized DataSource getDataSource () throws NamingException
	{
		if (initialContext == null)
		{
			initialContext = new InitialContext ();
		}

		return (DataSource) initialContext.lookup (
			"java:comp/env/jdbc/shipneeds2ConnectionPool");
	}
	
	protected void closeConnection (Connection con)
	{
		if (con != null)
		{				
			try { con.close (); } 
			catch (Exception e) {}
		}
	}
	
	protected void closeStatement (Statement stmt)
	{
		if (stmt != null)
		{
			try { stmt.close (); }
			catch (Exception e)
			{
				// TODO: handle this exception
			}
		}
	}
		
	protected void rollbackTransaction (Connection con)
	{
		if (con != null)
		{
			try { con.rollback (); }
			catch (Exception e1)
			{
				// TODO: handle this exception
			}
		}
	}

	public void deleteRowsByIds (String table, long[] ids) throws TransactionException
	{
		Connection con = getConnection ();
		PreparedStatement stmt = null;
		
		try
		{
			StringBuffer sql = new StringBuffer("DELETE FROM ");
			sql.append (table);
			sql.append (" WHERE ");

			for (int i = 0; i < ids.length; i++)
			{
				if (i > 0) sql.append(" OR ");
				sql.append ("id=");
				sql.append (Long.toString (ids[i]));
			}
			
			stmt = con.prepareStatement (sql.toString ());
		
			stmt.executeUpdate ();
			con.commit ();			
		}
		catch (Exception e)
		{
			rollbackTransaction (con);
			throw new TransactionException (e.getMessage (), e.getCause ());
		}
		finally
		{
			closeStatement (stmt);
			closeConnection (con);
		}
	}

	public void setColumnActiveByIds (String table, long[] ids, boolean active) throws TransactionException
	{
		Connection con = getConnection ();
		PreparedStatement stmt = null;
		
		try
		{
			StringBuffer sql = new StringBuffer("UPDATE ");
			sql.append (table);
			sql.append (" SET active=? WHERE ");

			for (int i = 0; i < ids.length; i++)
			{
				if (i > 0) sql.append(" OR ");
				sql.append ("id=");
				sql.append (Long.toString (ids[i]));
			}
			
			stmt = con.prepareStatement (sql.toString ());
			stmt.setBoolean (1, active);
		
			stmt.executeUpdate ();
			con.commit ();			
		}
		catch (Exception e)
		{
			rollbackTransaction (con);
			throw new TransactionException (e.getMessage (), e.getCause ());
		}
		finally
		{
			closeStatement (stmt);
			closeConnection (con);
		}
	}	
}
