/*
 * $Id: TableRowSelect.java,v 1.1 2007/09/11 11:24:22 bacon Exp $
 */

package com.unimar.shipneeds.view;

import com.sun.data.provider.RowKey;
import com.sun.rave.web.ui.event.TableSelectPhaseListener;
import java.util.TreeSet;
import javax.faces.context.FacesContext;
import javax.faces.el.ValueBinding;


public class TableRowSelect
{
	private TableSelectPhaseListener tspl = null; // Phase listener.
	private String bindingName;
	private TreeSet selectedRows;
	
	// Default constructor.
	public TableRowSelect (String rowGroupSourceName)
	{
		bindingName = "#{" + rowGroupSourceName + ".tableRow}";
		tspl = new TableSelectPhaseListener ();
		selectedRows = new TreeSet();
	}
	
	// Clear selected state from phase listener (e.g., when deleting rows).
	public void clear ()
	{
		tspl.clear ();
		selectedRows.clear ();
	}
	
	// Test flag indicating that selected objects should not be cleared.
	public boolean isKeepSelected ()
	{
		return tspl.isKeepSelected ();
	}
	
	// Set flag indicating that selected objects should not be cleared.
	public void keepSelected (boolean keepSelected)
	{
		tspl.keepSelected (keepSelected);
	}
	
	// Get selected property.
	public Object getSelected ()
	{
		return tspl.getSelected (getTableRow ());
	}
	
	// Set selected property.
	public void setSelected (Object object)
	{
		RowKey rowKey = getTableRow ();
		if (rowKey != null)
		{					
			tspl.setSelected (rowKey, object);
			
			if (object instanceof Boolean)
			{
				Boolean b = (Boolean)object;
				if (b.booleanValue ())
				{
					selectedRows.add (rowKey.getRowId ());
				}
				else
				{
					selectedRows.remove (rowKey.getRowId ());
				}
			}
		}
	}
	
	// Get selected value property.
	public Object getSelectedValue ()
	{
		RowKey rowKey = getTableRow ();
		return (rowKey != null) ? rowKey.getRowId () : null;
	}
	
	// Get the selected state -- Sort on checked state only.
	public boolean getSelectedState ()
	{
		// Typically, selected state is tested by comparing the selected and
		// selectedValue properties. In this example, however, the phase
		// listener value is not null when selected.
		return getSelectedState (getTableRow ());
	}
	
	// Get the selected state.
	public boolean getSelectedState (RowKey rowKey)
	{
		return tspl.isSelected (rowKey);
	}
	
	// Get current table row.
	//
	// Note: To obtain a RowKey for the current table row, the use the same
	// sourceVar property given to the TableRowGroup component. For example, if
	// sourceVar="name", use "#{name.tableRow}" as the expression string.
	private RowKey getTableRow ()
	{
		FacesContext ctx = FacesContext.getCurrentInstance ();
		ValueBinding vb = ctx.getApplication ().createValueBinding (bindingName);
		//"#{currentRow.tableRow}");
		return (RowKey) vb.getValue (ctx);
	}
	
	public String[] getSelectedRowIds ()
	{
		String[] buf = new String[selectedRows.size ()];
		return (String[])selectedRows.toArray (buf);
	}
}